// File: libcfg.h
// Author: Ivan V. Kulkov
// 24.02.2007
// Copyrigtht (C) 2007.
// Confguration registry library definitions

#ifndef KIV_LIBLOG_INTERFACE_HEADER
#define KIV_LIBLOG_INTERFACE_HEADER

//
// Result codes
//

enum
{
	LIBCFG_SUCCESS 				= 0,		// Operatin sucessfuly completed
	LIBCFG_ERROR_BASE_AND_NAME_TOO_LONG 	= -1,		// Total length of base and name parameters too long
	LIBCFG_ERROR_OPEN_FOR_READ		= -2,		// Error while opening file for reading
	LIBCFG_ERROR_OPEN_FOR_WRITE		= -3,		// Error while opening file for writing
	LIBCFG_ERROR_READING			= -4,		// Error while reading parameter from file
	LIBCFG_ERROR_WRITING			= -5,		// Error while writing parameter to file
	LIBCFG_ERROR_SEEKING_TO_END		= -6,		// Error while looking for file end
	LIBCFG_ERROR_PARAMETER_SIZE_TOO_SMALL	= -7,		// Parameter is too small for requested data type 
	LIBCFG_ERROR_PARAMETER_SIZE_ZERO	= -8,		// Parameter size if 0
	LIBCFG_ERROR_PARAMETER_SIZE_TOO_BIG	= -9,		// Parameter is too big for requested data type
	LIBCFG_ERROR_UNDEFINED_LENGTH		= -10,		// Undefined length argument
	LIBCFG_ERROR_UNDEFINED_BASE		= -11,		// Undefined base path argument
	LIBCFG_ERROR_UNDEFINED_NAME		= -12,		// Undefined parameter name argument
	LIBCFG_ERROR_UNTERMINATED_STRING	= -13,		// String parameter lack for end of string flag
	LIBCFG_ERROR_BAD_PATH			= -14,		// Bad path defined in base of path argument
	LIBCFG_ERROR_CREATING_DIRECTORY		= -15		// Error while creating parameter path directory
};
enum 
{
	MAX_PATH				= 512
};


#ifdef __cplusplus
extern "C" {
#endif

//
// Get parameter functions
//

int get_param_c(const char* base, const char* name, char* value);
int get_param_uc(const char* base, const char* name, unsigned char* value);

int get_param_s(const char* base, const char* name, short* value);
int get_param_us(const char* base, const char* name, unsigned  short* value);

int get_param_i(const char* base, const char* name, int * value);
int get_param_ui(const char* base, const char* name, unsigned int * value);

int get_param_l(const char* base, const char* name, long * value);
int get_param_ul(const char* base, const char* name, unsigned long * value);

int get_param_ll(const char* base, const char* name, long long * value);
int get_param_ull(const char* base, const char* name, unsigned long long * value);

int get_param_str(const char* base, const char* name, char* value, unsigned long max_len);
int get_param_data(const char* base, const char* name, char* value, unsigned long length);

//
// Set parameter functions
//

int set_param_c(const char* base, const char* name, char value);
int set_param_uc(const char* base, const char* name, unsigned char value);

int set_param_s(const char* base, const char* name, short value);
int set_param_us(const char* base, const char* name, unsigned  short value);

int set_param_i(const char* base, const char* name, int value);
int set_param_ui(const char* base, const char* name, unsigned int value);

int set_param_l(const char* base, const char* name, long value);
int set_param_ul(const char* base, const char* name, unsigned long value);

int set_param_ll(const char* base, const char* name, long long value);
int set_param_ull(const char* base, const char* name, unsigned long long value);

int set_param_str(const char* base, const char* name, char* value);
int set_param_data(const char* base, const char* name, char* value, unsigned long len);


#ifdef __cplusplus
}
#endif

#endif //KIV_LIBLOG_INTERFACE_HEADER
