#include "unixsocket.h"

UnixSocket::UnixSocket()
{

}

bool UnixSocket::bind(const char *addr_str)
{
    unlink(addr_str);
    bzero(&m_serv_addr, sizeof(m_serv_addr));
    m_serv_addr.sun_family = AF_UNIX;
    strcpy(m_serv_addr.sun_path, addr_str);
    m_saddrlen = sizeof(m_serv_addr.sun_family) + strlen(m_serv_addr.sun_path);

    if (::bind(sockfd, (struct sockaddr *)&m_serv_addr, m_saddrlen) < 0)
    {
        perror("error of bind");
        return false;
    }

    return true;
}

int UnixSocket::readDatagram(char *buf, int len)
{
    int n;
    if ((n = read(sockfd, buf, len)) < 0)
        perror("error of read");

    return n;
}

int UnixSocket::writeDatagram(const char* buf, int len, const char *clnt_addr_str)
{
    struct sockaddr_un clnt_addr;
    clnt_addr.sun_family = AF_UNIX;
    strcpy(clnt_addr.sun_path, clnt_addr_str);

    int clnt_addr_len = sizeof(clnt_addr);

    int n;

    if ( (n = (int)sendto(sockfd, buf, len, 0, (struct sockaddr *)&clnt_addr, (socklen_t)clnt_addr_len)) < 0)
        perror("error of sendto");

    return n;
}

int UnixSocket::send(const char* buf, int len, const char *clnt_addr_str)
{
    int s = socket(AF_UNIX, SOCK_DGRAM, 0);
    if(s<=0) return s;
    
    struct sockaddr_un clnt_addr;
    clnt_addr.sun_family = AF_UNIX;
    strcpy(clnt_addr.sun_path, clnt_addr_str);

    int send = (int)sendto(s, buf, len, 0, (struct sockaddr *)&clnt_addr, (socklen_t)sizeof(clnt_addr));
    
    close(s);
    
    return send;
}

bool UnixSocket::init()
{    
    if ((sockfd = socket(AF_UNIX, SOCK_DGRAM, 0)) < 0)
    {
        perror("error of socket");
        return false;
    }

    return true;
}

void UnixSocket::finish()
{
    close(sockfd);
}

int UnixSocket::getSockfd()
{
    return sockfd;
}

int UnixSocket::bytesAvailable()
{
    size_t bytes;
    if (ioctl(sockfd, FIONREAD, (char *) &bytes) < 0)
    {
        perror("error of ioctl");
        return -1;
    }

    return (int)bytes;
}
