#ifndef PSYNCCONFIG_H
#define PSYNCCONFIG_H

#include <QDialog>
#include <QMouseEvent>
#include <QKeyEvent>
#include <QFileDialog>
#include <QDir>
#include <QLineEdit>
#include <QSocketNotifier>
#include <QProcess>
#include <QDateTime>
#include <QMessageBox>
#include <QCursor>
#include <kuser.h>
#include <klocale.h>
#include <kglobal.h>
#include "framelogin.h"
#include "ipcpacket.h"
#include "ipc_interface.h"
#include "unixsocket.h"
#include "libcfg.h"
#include "ui_psync_config.h"
#include <kcmodule.h>

namespace Ui {
    class PSyncConfig;    
}

class PSyncConfig : public KCModule, public Ui::PSyncConfig
{
    Q_OBJECT

public:
    explicit PSyncConfig(QWidget *parent, const QVariantList &args);
    ~PSyncConfig();    

//protected:
  //  bool eventFilter(QObject *obj, QEvent *ev);

private slots:
    void syncIntervalsChanged(int index);
    void setLineEditText(int index);
    void setComboBoxEnable(int state);
    void callFileDialog();
    void openAccountInBrowser();
    void syncronizeStartStop();
    void receiveNotification();
    void onLoginButton();
    void settingsChanged();
    void sendSyncronizeIs();
    void showTrayIconChanged();

    void on_m_button_logout_clicked();

    void on_m_btn_addobject_clicked();

    void on_m_btn_delobject_clicked();

    void on_tabWidget_currentChanged(int index);

private:
    inline void initComboBox();
    inline void createComboBoxsLineEdit();
    inline void readSettings();        
    inline void saveSettings();
    inline void syncSettingsInstance();
    inline void initSocket();
    inline void setUserPixmap();
    inline void setLastSyncNote(const QDateTime &lastsync);// = QDateTime());
    //inline void check_request_lastsync_from_config();
    inline void initFrameLogin();

    void updateSyncInfo();
    void checkPassword(QString login, QString password);
    void showPasswordRequest();
    void showRegularView();
    void updateQuotaInfo();
    void setQuoteTotal(unsigned long long bytes);
    void setQuoteUsed(unsigned long long bytes);
    void initMargins();
    void sendSettingsChangedNotify();    
    unsigned long fromIndexToSec(int index);
    int fromSecToIndex(unsigned long Sec);

private:
    void updateRegularView();

    Ui::PSyncConfig *ui;    
    QLineEdit *m_ln;
    QLineEdit *m_svpnt;
    QPushButton *m_login_button;
    QPushButton *m_create_account_button;
    FrameLogin *frmlgn;   

    bool m_sync_is_running;
    bool m_request_lastsync_from_config;

    UnixSocket *m_us;
    QSocketNotifier *m_sn;
    Qt::WindowFlags wf;

    QDateTime m_datetime_lastsync;

    QString m_login;
    QString m_password;
};

#endif // PSYNCCONFIG_H
