#include "../libpsync/include/sync_global.h"
#include "../psyncnotify/include/ipc_interface.h"
#include "../syncd/include/ipc_interface.h"

#include "psyncconfig.h"
#include "config.h"
//#include "ui_psyncconfig.h"

#include "ipc_interface.h"
#include "ipc_interface_local.h"

#include <ipcmedia.h>

// include notifier programm IPC API


#include <QDesktopWidget>

#include <kpluginfactory.h>

#define DEFAULT_FOLDER_STR_SEPARATOR ": "

#define _SERVICE_NAME 				i18n(GLOBAL_SERVICE_PRODUCT_NAME)
#define _PROGRAMM_NAME 				i18n("%1 configuration",QString(GLOBAL_SERVICE_PRODUCT_NAME))

K_PLUGIN_FACTORY(factory, registerPlugin<PSyncConfig>();)
K_EXPORT_PLUGIN(factory("psyncconfig", "psyncconfig"))

PSyncConfig::PSyncConfig(QWidget *parent, const QVariantList &args) :
KCModule(factory::componentData(),parent,args),
    ui(new Ui::PSyncConfig)
{
    Buttons btn = 0;
    setButtons(btn);
    m_login = "";
    m_password = "";
    m_sync_is_running=false;
    ui->setupUi(this);
    
    if(QFile::exists(PSYNCCONFIG_IPC_NAME))
    {
    	int rc = IPCMedia::send(PSYNCCONFIG_IPC_NAME,PSYNCCONFIG_IPC_NAME,PSYNCCONFIG_ACTIVATE_WINDOW);
    	qDebug("STARTRC=%d",rc);
    	if(rc==0) exit(0);
    	QFile::remove(PSYNCCONFIG_IPC_NAME);
    }

    initFrameLogin();
    initComboBox();
    createComboBoxsLineEdit();

    //ui->comboBox_syncronize_interval->installEventFilter(this);
    connect(ui->checkBox_is_syncronized, SIGNAL(stateChanged(int)), this, SLOT(setComboBoxEnable(int)));
    connect(ui->pushButton_change_folder, SIGNAL(clicked()), this, SLOT(callFileDialog()));    
    connect(ui->pushButton_parameters, SIGNAL(clicked()), this, SLOT(openAccountInBrowser()));

    syncSettingsInstance();
    readSettings();

    setLineEditText(ui->comboBox_syncronize_interval->currentIndex());
    setUserPixmap();

    initSocket();
    connect(ui->pushButton_syncronize, SIGNAL(clicked()), this, SLOT(syncronizeStartStop()) );

    //initMargins();
    connect(ui->checkBox_is_syncronized, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()));
    connect(ui->checkBox_is_syncronized, SIGNAL(stateChanged(int)), this, SLOT(sendSyncronizeIs()));

    connect(ui->comboBox_syncronize_interval, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
    connect(ui->m_cb_showTrayIcon, SIGNAL(stateChanged(int)), this, SLOT(showTrayIconChanged()));
    
    ui->pushButton_save->setVisible(false);
    ui->pushButton_restore->setVisible(false);
    ui->pushButton_clear->setVisible(false);
    ui->pushButton_new_save_point->setVisible(false);
    ui->comboBox_save_point->setVisible(false);
    ui->label_save_date->setVisible(false);
    ui->label_save_date_note->setVisible(false);
    ui->label_save_point_note->setVisible(false);
    ui->label_size->setVisible(false);
    ui->label_size_notr->setVisible(false);
    ui->label_new_save_point_note->setVisible(false);
    ui->label_name_save_point->setVisible(false);
    ui->lineEdit_name_save_point->setVisible(false);
    ui->line->setVisible(false);

    if(!QFile::exists(PSYNCD_IPC_NAME)) {
        QMessageBox::critical(this, _SERVICE_NAME, i18n("MandrivaSync service is not running. Please contact your system administrator."), QMessageBox::Ok);
        exit(-1);
    }

    IPCMedia::send(PSYNCD_IPC_NAME, PSYNCCONFIG_IPC_NAME, PSYNCD_GET_SYNC_STATUS);


    if (m_password.isEmpty())
    {
        showPasswordRequest();
    }
    else
    {
        checkPassword(m_login, m_password);
    };
    updateRegularView();

}

PSyncConfig::~PSyncConfig()
{
    saveSettings();    
    m_us->finish();
    QFile::remove(PSYNCCONFIG_IPC_NAME);
    delete ui;
    delete m_us;
}

/*void PSyncConfig::show()
{
	setWindowTitle(_PROGRAMM_NAME);
	QDialog::show();
}
*/
void PSyncConfig::initFrameLogin()
{
    frmlgn = new FrameLogin(ui->tab_account);
    frmlgn->setFrameStyle(QFrame::Plain);        
    frmlgn->setVisible(false);    

    ui->tab_account->layout()->addWidget(frmlgn);
    m_login_button = frmlgn->getButtonLogin();
    m_create_account_button = frmlgn->getButtonCreateAccount();
    m_create_account_button->setAutoFillBackground(true);
    m_create_account_button->setStyleSheet("  border: none; color: blue; ");

    connect(m_login_button, SIGNAL(clicked()), this, SLOT(onLoginButton()));
    connect(m_create_account_button, SIGNAL(clicked()), this, SLOT(openAccountInBrowser()));
}

void PSyncConfig::syncIntervalsChanged(int index)
{
    setLineEditText(index);
}

void PSyncConfig::setLineEditText(int index)
{        
    m_ln->setText(ui->comboBox_syncronize_interval->itemText(index));
}

void PSyncConfig::initComboBox()
{
    ui->comboBox_syncronize_interval->insertSeparator(ui->comboBox_syncronize_interval->count() - 1);
    connect(ui->comboBox_syncronize_interval, SIGNAL(activated(int)), this, SLOT(syncIntervalsChanged(int)));
    connect(ui->comboBox_syncronize_interval, SIGNAL(highlighted(int)), this, SLOT(setLineEditText(int)));
    ui->comboBox_syncronize_interval->setInsertPolicy(QComboBox::NoInsert);        
}

void PSyncConfig::createComboBoxsLineEdit()
{
    m_ln = new QLineEdit();
    m_ln->setReadOnly(true);
    ui->comboBox_syncronize_interval->setLineEdit(m_ln);

    m_svpnt = new QLineEdit;
    m_svpnt->setReadOnly(true);
    ui->comboBox_save_point->setLineEdit(m_svpnt);
}
/*
bool PSyncConfig::eventFilter(QObject *obj, QEvent *event)
{*/
/*    if ((obj == ui->comboBox_syncronize_interval) && (event->type() == QEvent::KeyPress))
    {
        QKeyEvent *ke = (QKeyEvent*)event;
        if (ke->key() == Qt::Key_Space)
        {
            QMouseEvent *me = new QMouseEvent(QEvent::MouseButtonPress,
            		QPoint(ui->comboBox_syncronize_interval->width() - 5, ui->comboBox_syncronize_interval->height() - 5),
            		Qt::LeftButton, Qt::LeftButton, Qt::NoModifier);
            QCoreApplication::postEvent(ui->comboBox_syncronize_interval, me);
        }
    }    

    */
/*	return QDialog::eventFilter(obj, event);
}
*/
void PSyncConfig::setComboBoxEnable(int state)
{
    qDebug("state = %d", state);
        ui->comboBox_syncronize_interval->setEnabled(state);
        ui->pushButton_syncronize->setEnabled(state);
}

void PSyncConfig::readSettings()
{
    const int size_value = 512;
    char value[size_value];
    bool correct = true;
    bool correct_quota = true;
    unsigned long interval = 0;

    if (get_param_str(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_SYNC_IS, value, size_value) == LIBCFG_SUCCESS)
        ui->checkBox_is_syncronized->setCheckState((Qt::CheckState)QVariant(value).toInt());
    else
        correct = false;


    get_param_ul(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_SYNC_INTERVAL, &interval);
    ui->comboBox_syncronize_interval->setCurrentIndex(fromSecToIndex(interval));


    QString default_folder;
    if (get_param_str(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_DEFAULT_FOLDER, value, size_value) == LIBCFG_SUCCESS)
    {
        if (value[0] == ' ')
            default_folder = QDir::homePath()+"/sync-unresolved";
        else
            default_folder = QVariant(QString::fromLocal8Bit(value)).toString();
    }
    else
        correct = false;

    ui->label_current_folder->setText(ui->label_current_folder->text() + default_folder);

    unsigned int time=0;
    get_param_ui(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_LAST_SYNC_TIME, &time);
    if(time)
    {
        QDateTime dt;
        dt.setTime_t(time);
        setLastSyncNote(dt);
    }
    //else
      //  correct = false;

    // Read tray bar icon settings
    if (get_param_str(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_SHOW_TRAY, value, size_value) == LIBCFG_SUCCESS)
        ui->m_cb_showTrayIcon->setCheckState((Qt::CheckState)QVariant(value).toInt());
    else
        correct = false;

    unsigned long long quota;
    unsigned long long quota_u;
    if (get_param_ull(GLOBAL_SERVICE_INFO_DIR, SETTINGS_QUOTA_TOTAL, &quota) == LIBCFG_SUCCESS)
    {
        setQuoteTotal(quota);
    }
    else
        correct_quota = false;    

    if (get_param_ull(GLOBAL_SERVICE_INFO_DIR, SETTINGS_QUOTA_USED, &quota_u) == LIBCFG_SUCCESS)
    {
        setQuoteUsed(quota_u);
    }
    else
        correct_quota = false;    

    if (correct_quota)
        ui->progressBar->setToolTip(i18n("Used %1 Mb. of available %2 Mb.", quota_u / 1000 / 1000, quota / 1000 / 1000));

    if (get_param_str(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_LOGIN, value, size_value) == LIBCFG_SUCCESS)
    {
	m_login = value;
	ui->label_account->setText(value);
	frmlgn->setLogin(value);
    }
    else return;
    

    if (get_param_str(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_PASSWORD, value, size_value) == LIBCFG_SUCCESS)
        m_password = QVariant(value).toString();
    else return;
    

    if (!correct)
        QMessageBox::critical(this, _SERVICE_NAME, i18n("Error reading configuration"), QMessageBox::Ok);
}

void PSyncConfig::saveSettings()
{
	int rc;

    if (set_param_str(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_SYNC_IS, QVariant(ui->checkBox_is_syncronized->checkState()).toString().toLocal8Bit().data()) != LIBCFG_SUCCESS)
    {
        QMessageBox::critical(this, _SERVICE_NAME, i18n("Error saving settings"), QMessageBox::Ok);
    }
        
    if (set_param_str(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_SHOW_TRAY, QVariant(ui->m_cb_showTrayIcon->checkState()).toString().toLocal8Bit().data()) != LIBCFG_SUCCESS)
    {
        QMessageBox::critical(this, _SERVICE_NAME, i18n("Error saving settings"), QMessageBox::Ok);
    }

    unsigned long interval = fromIndexToSec(ui->comboBox_syncronize_interval->currentIndex());
    if (set_param_ul(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_SYNC_INTERVAL, interval) != LIBCFG_SUCCESS)
    {
        QMessageBox::critical(this, _SERVICE_NAME, i18n("Error saving settings"), QMessageBox::Ok);
    }

    QStringList sl = ui->label_current_folder->text().split(QString(DEFAULT_FOLDER_STR_SEPARATOR));
    if (set_param_str(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_DEFAULT_FOLDER, sl[1].toLocal8Bit().data()) != LIBCFG_SUCCESS)
    {
        QMessageBox::critical(this, _SERVICE_NAME, i18n("Error saving settings"), QMessageBox::Ok);
    }

//    if (!m_login.isEmpty())
    //{
    rc = set_param_str(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_LOGIN, QVariant(m_login).toString().toLocal8Bit().data());
    	if (rc!= LIBCFG_SUCCESS)
        {
            QMessageBox::critical(this, _SERVICE_NAME, i18n("Error saving settings")+QString(" (%1)").arg(rc), QMessageBox::Ok);
        }
    rc = set_param_str(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_PASSWORD, QVariant(m_password).toString().toLocal8Bit().data());
        if (rc!= LIBCFG_SUCCESS)
        {
            QMessageBox::critical(this, _SERVICE_NAME, i18n("Error saving settings")+QString(" (%1)").arg(rc), QMessageBox::Ok);
        }
//    }

    sendSettingsChangedNotify();
}

void PSyncConfig::checkPassword(QString login, QString password)
{
    qDebug("login = %s, password = %s", login.toLocal8Bit().data(), password.toLocal8Bit().data());

    if ((login.isEmpty()) || (password.isEmpty()))
    {
        QMessageBox::warning(this, _SERVICE_NAME, i18n("Need to enter your login and password of nonzero length"), QMessageBox::Ok);
        return ;
    }

	QString info = login + ":" + password;
	IPCPacket* pkt = IPCPacket::Create(REQUEST_LOGIN_AND_PASSWORD, (char*)CONFIG_PROGRAM_NAME, (unsigned char*)info.toLocal8Bit().data(), (unsigned long)info.length());

	QString receiver_address(PSYNCD_IPC_NAME);
	if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
	{
		qDebug("IPC error with %s: type %lu",receiver_address.toLocal8Bit().data(),pkt->getType());
		//QMessageBox::critical(this, i18n("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
	}

	if (pkt) delete pkt;

}

void PSyncConfig::showPasswordRequest()
{
    m_password = "";
    m_login = "";
    ui->tab_objects->setEnabled(false);
    ui->tab_settings->setEnabled(false);
    ui->frame_save_point->setVisible(false);
    frmlgn->setLogin("");
    frmlgn->setPassword("");
    frmlgn->show();
}

void PSyncConfig::syncSettingsInstance()
{

}

void PSyncConfig::callFileDialog()
{
	QStringList pe = QProcess::systemEnvironment();

	QStringList sl = ui->label_current_folder->text().split(QString(DEFAULT_FOLDER_STR_SEPARATOR));

	QString sf = QFileDialog::getExistingDirectory(this, i18n("Folder to save files by default"), sl[1], QFileDialog::ShowDirsOnly);

	if (!sf.isEmpty())
	{
		ui->label_current_folder->setText(sl[0] + QString(": ") + sf);
		this->saveSettings();
	}
}

void PSyncConfig::openAccountInBrowser()
{
    QString program = "/usr/bin/www-browser";
    QStringList arguments;
    arguments << "https://www.mandrivasync.com";
    QProcess* process = new QProcess(this);
    process->start(program, arguments);
}

///
/// Syncronisation start/stop button slot
///

void PSyncConfig::syncronizeStartStop()
{
    QString button_text;
    IPCPacket * pkt = NULL;

    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    ui->pushButton_syncronize->setEnabled(false);

    if (m_sync_is_running)
    {
		pkt = IPCPacket::Create(PSYNCD_SYNC_STOP, (char*)PSYNCCONFIG_IPC_NAME,0,0);
		m_sync_is_running = false;
    }
    else
    {
		pkt = IPCPacket::Create(PSYNCD_SYNC_START, (char*)PSYNCCONFIG_IPC_NAME,0,0);
		m_sync_is_running = true;
    }

	if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), PSYNCD_IPC_NAME) < 0)
    {
		QApplication::restoreOverrideCursor();
		m_sync_is_running=!m_sync_is_running;
		qDebug("IPC error with %s: type %lu",PSYNCD_IPC_NAME,pkt->getType());
        //QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
    }
    
    if(pkt) delete pkt;

    updateRegularView();
}

///
/// IPC Socket creation
///

void PSyncConfig::initSocket()
{
    m_us = new UnixSocket();
    if (m_us->init())
    {
        qDebug("sockfd = %d", m_us->getSockfd());
        QString self_address(PSYNCCONFIG_IPC_NAME);
        QFileInfo fi(PSYNCCONFIG_IPC_NAME);
        if (!fi.absoluteDir().exists())
        	fi.absoluteDir().mkdir(PSYNCCONFIG_IPC_NAME);

        if (m_us->bind(self_address.toLocal8Bit().data()))
        {
            m_sn = new QSocketNotifier(m_us->getSockfd(), QSocketNotifier::Read);
            connect(m_sn , SIGNAL(activated(int )), this, SLOT(receiveNotification()));
        }
        else
        {
            qDebug("Error of local socket bind");//TODO//
            QMessageBox::critical(this, _SERVICE_NAME, i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
        }
    }
    else
    {
        qDebug("Error of local socket init");
        QMessageBox::critical(this, _SERVICE_NAME, i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
    }

    m_sync_is_running = false;
}

void PSyncConfig::onLoginButton()
{
    m_login = frmlgn->getLogin();
    m_password = frmlgn->getPassword();
    checkPassword(m_login, m_password);
    if (!(m_login.isEmpty()) && !(m_password.isEmpty())){
        QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    }
}

void PSyncConfig::on_m_button_logout_clicked()
{
	int ret = QMessageBox::question(this, _SERVICE_NAME, i18n("After log out current account information will be lost!\nPlease ensure you remember login and password.\nDo you really want to log out?"),QMessageBox::Yes|QMessageBox::No);

	if(ret == QMessageBox::No) return;

	showPasswordRequest();
	ui->checkBox_is_syncronized->setChecked(false);
}

void PSyncConfig::receiveNotification()
{    
    IPCPacket *m_pkt = NULL;
    QString res;
    QStringList all_objects;

    int total_bytes = m_us->bytesAvailable();
    if (m_pkt)
        return;

    m_pkt = new IPCPacket();
    m_us->readDatagram((char*)m_pkt->readBuffer(total_bytes), total_bytes);    

    if (m_pkt->IsValid())
    {
        switch (m_pkt->getType())
        {
        	case PSYNCCONFIG_SYNC_START:
        		ui->pushButton_syncronize->setEnabled(true);
        	case PSYNCD_ANS_SYNC_STATUS_ONLINE:
				m_sync_is_running = true;
				QApplication::restoreOverrideCursor();
				updateRegularView();
				break;
			case PSYNCCONFIG_SYNC_STOP:
				ui->pushButton_syncronize->setEnabled(true);
			case PSYNCD_ANS_SYNC_STATUS_OFFLINE:
				m_sync_is_running = false;
				QApplication::restoreOverrideCursor();
                updateRegularView();
				break;
            case PSYNCCONFIG_ACTIVATE_WINDOW:
            	{
					QRect d = QApplication::desktop()->screenGeometry();
					QRect w = geometry();
					move((d.width()-w.width())/2,(d.height()-w.height())/2);
					raise();
					activateWindow();
            	}
                break;
            case SETTINGS_CLOSE_COMMAND:
                close();
                break;
            case PSYNCCONFIG_ACCOUNT_INVALID:
                QApplication::restoreOverrideCursor();
            	QMessageBox::critical(this, _SERVICE_NAME, i18n("Account name or password is invalid"), QMessageBox::Ok);
                showPasswordRequest();
                break;
            case PSYNCCONFIG_ACCOUNT_VALID:
                showRegularView();
                break;
            case PSYNCD_OBJECTS_LIST:
            	res = QString::fromLocal8Bit((QByteArray((const char*)m_pkt->getData(),m_pkt->getDataLen())));
            	//if(res.length()==0) break;
            	all_objects = res.split("\n",QString::SkipEmptyParts);
            	ui->m_list_objects->clear();
                ui->m_list_objects->addItems(all_objects);
                updateRegularView();
                break;

            default:
                qDebug("Unknown type of message. The number of message is %d", (int)m_pkt->getType());
        }
    }

    if (m_pkt)
        delete m_pkt;
}

void PSyncConfig::updateSyncInfo()
{
    unsigned int time=0;
    get_param_ui(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_LAST_SYNC_TIME, &time);
    if(time)
    {
        QDateTime dt;
        dt.setTime_t(time);
        setLastSyncNote(dt);
    }
}


void PSyncConfig::updateRegularView()
{
    if(m_password=="") return;

    if (!m_sync_is_running)
    {
    	ui->pushButton_syncronize->setText(i18n("Synchronize now"));
        ui->checkBox_is_syncronized->setEnabled(true);
        ui->m_button_logout->setEnabled(true);
        ui->comboBox_syncronize_interval->setEnabled(true);
        ui->pushButton_change_folder->setEnabled(true);
        ui->tab_objects->setEnabled(true);
    }
    else
    {
    	ui->pushButton_syncronize->setText(i18n("Stop synchronization"));
        ui->checkBox_is_syncronized->setEnabled(false);
        ui->comboBox_syncronize_interval->setEnabled(false);
        ui->pushButton_change_folder->setEnabled(false);
        ui->m_button_logout->setEnabled(false);
        ui->tab_objects->setEnabled(false);
    }
    updateSyncInfo();
    updateQuotaInfo();
}

void PSyncConfig::setUserPixmap()
{
    KUser *user = new KUser();
    QString user_avatar_path = user->faceIconPath();
    qDebug("path = %s", user_avatar_path.toLocal8Bit().data());

    if (!user_avatar_path.isEmpty())
        ui->label_image->setPixmap(QPixmap(user->faceIconPath()).scaled(QSize(64,64)));
}

void PSyncConfig::setLastSyncNote(const QDateTime &lastsync)
{
    KLocale *aLocale = KGlobal::locale();

    if (lastsync.isNull())
        ui->label_last_syncronization->setText(i18n("The last synchronization: ") + aLocale->formatDateTime(QDateTime::currentDateTime(), KLocale::FancyShortDate).toLower());
    else
        ui->label_last_syncronization->setText(i18n("The last synchronization: ") + aLocale->formatDateTime(lastsync, KLocale::FancyShortDate).toLower());
}



void PSyncConfig::showRegularView()
{
    QApplication::restoreOverrideCursor();
    frmlgn->setVisible(false);
    ui->frame_save_point->setVisible(true);
    ui->label_account->setText(m_login);
    ui->tab_objects->setEnabled(true);
    ui->tab_settings->setEnabled(true);
    updateRegularView();
}

void PSyncConfig::updateQuotaInfo()
{
    unsigned long long quota;
    unsigned long long quota_u;
    bool correct_quota = true;    
    if (get_param_ull(GLOBAL_SERVICE_INFO_DIR, SETTINGS_QUOTA_TOTAL, &quota) == LIBCFG_SUCCESS)
    {
        setQuoteTotal(quota);
    }
    else
        correct_quota = false;    

    if (get_param_ull(GLOBAL_SERVICE_INFO_DIR, SETTINGS_QUOTA_USED, &quota_u) == LIBCFG_SUCCESS)
    {
        setQuoteUsed(quota_u);
    }
    else
        correct_quota = false;    

    if (correct_quota)
        ui->progressBar->setToolTip(i18n("Used %1 Mb. of available %2 Mb.", quota_u / 1000 / 1000, quota / 1000 / 1000));
        
    if (set_param_str(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_LOGIN, QVariant(m_login).toString().toLocal8Bit().data()) != LIBCFG_SUCCESS)
    {
	QMessageBox::critical(this, _SERVICE_NAME, i18n("Error saving settings"), QMessageBox::Ok);
    }

    if (set_param_str(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_PASSWORD, QVariant(m_password).toString().toLocal8Bit().data()) != LIBCFG_SUCCESS)
    {
	QMessageBox::critical(this, _SERVICE_NAME, i18n("Error saving settings"), QMessageBox::Ok);
    }
};



void PSyncConfig::setQuoteTotal(unsigned long long bytes)
{
    unsigned long long full = bytes / 1000 / 1000;
    //unsigned long long half = full / 2;

    ui->progressBar->setMaximum(full);
    ui->label_full->setText(QString::number(full) + i18n(" Mb."));
    //ui->label_half->setText(QString::number(half) + i18n(" Mb."));
}

void PSyncConfig::setQuoteUsed(unsigned long long bytes)
{
    unsigned int value = bytes / 1000 / 1000;
    ui->progressBar->setValue(value);
    ui->label_half->setText(QString::number(value) + i18n(" Mb."));
}

void PSyncConfig::initMargins()
{
    QString padding = "padding: 5px 10px 5px 10px;";

    ui->pushButton_save->setStyleSheet(padding);
    ui->pushButton_restore->setStyleSheet(padding);
    ui->pushButton_clear->setStyleSheet(padding);
    ui->pushButton_parameters->setStyleSheet(padding);
    ui->pushButton_new_save_point->setStyleSheet(padding);
    ui->pushButton_syncronize->setStyleSheet(padding);
    ui->pushButton_change_folder->setStyleSheet(padding);
}

void PSyncConfig::sendSettingsChangedNotify()
{    
    IPCPacket* pkt = IPCPacket::Create(SETTINGS_CHANGED, (char*)PSYNCCONFIG_IPC_NAME, 0,0);

    if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), PSYNCD_IPC_NAME) < 0)
    {
    	qDebug("IPC error with %s: type %lu",PSYNCD_IPC_NAME,pkt->getType());
        //QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
    }

    if (pkt) delete pkt;
}

void PSyncConfig::settingsChanged()
{
    saveSettings();
}

///
/// Syncronization checkbox
///

void PSyncConfig::sendSyncronizeIs()
{
    qDebug("sendSyncronizeIs");

    if ((ui->checkBox_is_syncronized->checkState() == Qt::Checked))
        IPCMedia::send(PSYNCNOTIFY_IPC_NAME, PSYNCCONFIG_IPC_NAME, PSYNCNOTIFY_SYNC_ENABLED);
    else
        IPCMedia::send(PSYNCNOTIFY_IPC_NAME, PSYNCCONFIG_IPC_NAME, PSYNCNOTIFY_SYNC_DISABLED);


}


void PSyncConfig::showTrayIconChanged()
{
    qDebug("showTrayIconChanged");

    if (ui->m_cb_showTrayIcon->checkState() == Qt::Checked)
    {
        IPCMedia::send(PSYNCNOTIFY_IPC_NAME, PSYNCCONFIG_IPC_NAME,PSYNCNOTIFY_SHOW_TRAY_ICON);
    }
    else
    {
        IPCMedia::send(PSYNCNOTIFY_IPC_NAME, PSYNCCONFIG_IPC_NAME,PSYNCNOTIFY_HIDE_TRAY_ICON);
    };

    //    set_param_str(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_SHOW_TRAY, "");
    set_param_str(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_SHOW_TRAY, QVariant(ui->m_cb_showTrayIcon->checkState()).toString().toLocal8Bit().data());

}

unsigned long PSyncConfig::fromIndexToSec(int index)
{
    if (index == 0)
        return 900;

    if (index == 1)
        return 1800;

    if (index == 2)
        return 3600;

    return 0;
}

int PSyncConfig::fromSecToIndex(unsigned long Sec)
{
    if (Sec == 900)
        return 0;

    if (Sec == 1800)
        return 1;

    if (Sec == 3600)
        return 2;

    return 4;
}

void PSyncConfig::on_m_btn_addobject_clicked()
{
	QFileDialog dialog(this);
	dialog.setFileMode(QFileDialog::ExistingFile);
	QStringList fileNames;
	if (dialog.exec()) fileNames = dialog.selectedFiles();

	if(fileNames.empty())	return;
	QString s = fileNames.join("\n");
	qDebug("Adding files: %s",s.toLocal8Bit().data());


	IPCPacket * pkt = IPCPacket::Create(PSYNCD_ADD_TO_SYNC_COMMAND, (char*)PSYNCCONFIG_IPC_NAME, (unsigned char*)s.toLocal8Bit().data(), strlen((const char*)s.toLocal8Bit().data()));

	QString receiver_address(PSYNCD_IPC_NAME);


	if(m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
	{
		qDebug("Error adding files: %s",s.toLocal8Bit().data());
	}

	if (pkt) delete pkt;

	on_tabWidget_currentChanged(ui->tabWidget->currentIndex());
}

void PSyncConfig::on_m_btn_delobject_clicked()
{
	if(!ui->m_list_objects->currentItem()) return;

	QString s = ui->m_list_objects->currentItem()->text();

	int ret = QMessageBox::question(this, _SERVICE_NAME, i18n("Delete file %1 from syncronization?\n",s),QMessageBox::Ok| QMessageBox::Cancel);

	if(ret != QMessageBox::Ok) return;

	qDebug("Deleting files: %s",s.toLocal8Bit().data());

	IPCPacket * pkt = IPCPacket::Create(PSYNCD_REMOVE_TO_SYNC_COMMAND, (char*)PSYNCCONFIG_IPC_NAME, (unsigned char*)s.toLocal8Bit().data(), strlen((const char*)s.toLocal8Bit().data()));

	QString receiver_address(PSYNCD_IPC_NAME);

	if(m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
	{
		qDebug("Error Deleting file from psyncd");
	}

	if (pkt) delete pkt;

	on_tabWidget_currentChanged(ui->tabWidget->currentIndex());
}

void PSyncConfig::on_tabWidget_currentChanged(int index)
{
    if(index!=2) return;
    IPCPacket* pkt = IPCPacket::Create(PSYNCD_GET_ALL_FILES_COMMAND, (char*)PSYNCCONFIG_IPC_NAME,0,0);

    QString receiver_address(PSYNCD_IPC_NAME);

    if (m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), receiver_address.toLocal8Bit().data()) < 0)
    {
        qDebug("IPC error with %s: type %lu",receiver_address.toLocal8Bit().data(),pkt->getType());
        //QMessageBox::critical(this, QString("Sync Client"), i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
    }

    if (pkt) delete pkt;
}
