#ifndef PSYNCNOTIFY_H
#define PSYNCNOTIFY_H

#include <QtGui>
#include <QFile>
#include <QSocketNotifier>
#include <QMessageBox>
#include <QMovie>
#include <QProgressBar>
#include <string>
#include <map>
#include <kprocess.h>
#include <knotification.h>
#include <klocale.h>
#include <kglobal.h>
#include <kapplication.h>
#include "ipc_interface.h"
#include "ipc_interface_local.h"
#include "ipcpacket.h"
#include "unixsocket.h"
#include "libcfg.h"

#include "progressnotifyjob.h"

class ProgressNotifier
{
public:
	ProgressNotifier();
	~ProgressNotifier();
	void setText(QString text);
	void setPercent(int value);
	QWidgetAction * m_text_action;
	QWidgetAction * m_progress_action;
	QLabel*			m_label;
	QProgressBar*	m_progress;
};

class ProgressNotifierArea
{
public:
	ProgressNotifierArea();
	~ProgressNotifierArea();

	bool attachToMenu(QMenu* menu, QAction * before);
	void detach();

	bool add(QString name);
	void setText(QString name, QString text);
	void setPercent(QString name, int value);
	void remove(QString name);
	bool exist(QString name);
protected:

	QMap<QString,ProgressNotifier*> m_notifiers;
	QMenu * m_menu;
	QAction*		m_sep1;
	QAction*		m_sep2;
	QAction*		m_before;
};

class PSyncNotify : public QDialog
{
    Q_OBJECT

public:
    PSyncNotify(KApplication *app, QWidget *parent = 0);
    ~PSyncNotify();

private slots:
    void receiveNotification();
    void iconActivated(QSystemTrayIcon::ActivationReason reason);    
    void updateMenuPosAndSize();
    void syncronizeStartStop();
    void openSettings();
    void closeApp();
    void updateIcon();
    void Notify_Close();
    void updateLastSyncTimeLabel();

private:
    void createActions();
    void createTrayIcon();
    QLabel *createLabel(QWidget *widget, const QString &text, Qt::AlignmentFlag aligment, int margin);
    QFont createFont(const QString &name, int size, bool is_bold, bool is_italic);

    inline void initSocket();
    inline void readSettings();

    void setLastSyncTimeLabel(QString str);
    void reportSyncStart();
    void reportSyncStop();


private:
    QWidgetAction *nameprogramAction;
    QWidgetAction *lastsynctimeAction;
    QAction *syncrAction;
    QAction *opensettingsAction;    
    //std::map<QString,KNotification*> m_notificationMap;

    std::map<QString,ProgressNotifyJob*> m_notificationMap;

    ProgressNotifierArea m_na;

    QSystemTrayIcon *trayIcon;
    QMenu *trayIconMenu;
    QIcon *m_icon;

    UnixSocket *m_us;
    QSocketNotifier *m_sn;

    bool 	m_sync_on;
    bool 	m_sync_is_running;
    int 	m_show_tray_icon;

    KApplication *m_app;
    QMovie *m_gif;
};

#endif // PSYNCNOTIFY_H
