#include "../libpsync/include/sync_global.h"
#include "../psyncconfig/include/config.h"
#include "../syncd/include/ipc_interface.h"
#include "../psyncconfig/include/ipc_interface.h"

#include "ipcmedia.h"
#include "psyncnotify.h"
#include "ipc_interface.h"
#include "progressnotifyjob.h"

//#define _PROGRAMM_NAME 			i18n(GLOBAL_SERVICE_PRODUCT_NAME)
#define _SERVICE_NAME 			i18n(GLOBAL_SERVICE_PRODUCT_NAME)
#define _PROGRAMM_NAME 			i18n("%1 client", QString(GLOBAL_SERVICE_PRODUCT_NAME))
#define _NOTIFICATION_CAPTION	i18n(GLOBAL_SERVICE_PRODUCT_NAME)

using namespace std;


ProgressNotifier::ProgressNotifier()
{
	m_label = NULL;
	m_label = new QLabel("");
	m_label->setStyleSheet(QString(" QLabel { \
		     border: 0px solid grey;\
			 margin: 3px 6px 0px 6px;\
			 text-align: center;\
			background-color: transparent;\
		 }"));

	m_progress = NULL;
	m_progress = new QProgressBar();
	m_progress->setMinimum(0);
	m_progress->setMaximum(100);
	m_progress->setValue(0);
	m_progress->setStyleSheet(QString(" QProgressBar { \
	     border-radius: 5px;\
		 height: 20px;\
		 border: 0px solid grey;\
		 margin: 0px 6px 3px 6px;\
		 text-align: center;\
		background-color: transparent;\
	 }\
	 QProgressBar::chunk {\
	     background-color: #76b3e8;\
	     width: 20px;\
	 }"));

	m_text_action = NULL;
	m_text_action = new QWidgetAction(NULL);
	m_text_action->setDefaultWidget(m_label);

	m_progress_action = NULL;
	m_progress_action = new QWidgetAction(NULL);
	m_progress_action->setDefaultWidget(m_progress);
}

ProgressNotifier::~ProgressNotifier()
{
	if(m_progress_action) delete m_progress_action;
	if(m_text_action) delete m_text_action;

	m_text_action = NULL;
	m_label	= NULL;
	m_progress = NULL;
	m_progress_action = NULL;
}

void ProgressNotifier::setText(QString text)
{
	m_label->setToolTip(text);
	m_progress->setToolTip(text);
	if(text.length()>60) text=text.left(60)+"...";
	m_label->setText(text);

}

void ProgressNotifier::setPercent(int value)
{
	m_progress->setValue(value);
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////


ProgressNotifierArea::ProgressNotifierArea()
{
	m_menu=NULL;
	m_before = NULL;
	m_sep1=NULL;
	m_sep2=NULL;
}

ProgressNotifierArea::~ProgressNotifierArea()
{
	detach();
}

bool ProgressNotifierArea::attachToMenu(QMenu* menu, QAction * before)
{
	if(m_menu) return false;
	m_menu = menu;
	m_before = before;
	return true;
}

void ProgressNotifierArea::detach()
{
	if(!m_menu) return;
	m_menu->removeAction(m_sep1);

	for (QMap<QString,ProgressNotifier*>::iterator i = m_notifiers.begin(); i != m_notifiers.end(); ++i)
	{
	     m_menu->removeAction(m_notifiers[i.key()]->m_text_action);
	     m_menu->removeAction(m_notifiers[i.key()]->m_progress_action);
	}
	m_notifiers.clear();
}

bool ProgressNotifierArea::add(QString name)
{
	if(!m_menu) return false;
	if(m_notifiers[name]) return true;
	m_notifiers[name]= new ProgressNotifier();

	if(!m_sep1)
	{
		m_sep1 = m_menu->insertSeparator(m_before);
	}

	m_menu->insertAction(m_sep1,m_notifiers[name]->m_text_action);
	m_menu->insertAction(m_sep1,m_notifiers[name]->m_progress_action);
	return true;
}

void ProgressNotifierArea::setText(QString name, QString text)
{
	if(!m_notifiers[name]) return;
	m_notifiers[name]->setText(text);
}

void ProgressNotifierArea::setPercent(QString name, int value)
{
	if(!m_notifiers[name]) return;
	m_notifiers[name]->setPercent(value);
}

void ProgressNotifierArea::remove(QString name)
{
	if(!m_menu) return;
	if(!m_notifiers[name]) return;

	m_menu->removeAction(m_notifiers[name]->m_text_action);
	m_menu->removeAction(m_notifiers[name]->m_progress_action);
	delete m_notifiers[name];
	m_notifiers.remove(name);
}

bool ProgressNotifierArea::exist(QString name)
{
	if(!m_notifiers[name]) return false;
	return true;
}


/////////////////////////////////////////////////////////////////////////////////////////////////////////



PSyncNotify::PSyncNotify(KApplication *app,QWidget *parent)
    : QDialog(parent)
{
    m_app = app;
    m_sync_is_running 		= false;
    m_show_tray_icon 		= true;
    m_sync_on			= false;
    m_icon 			= new QIcon(QString(":/red_status.png"));
    m_gif 			= new QMovie(":animation.gif");

    setWindowIcon(*m_icon);
    readSettings();

    createActions();
    initSocket();
    createTrayIcon();

    // Setup GUI state according to readed config
    updateLastSyncTimeLabel();

    if(m_sync_on)
    {
	qDebug("Sync enabled!!!");
	syncrAction->setEnabled(true);
	trayIcon->setIcon(QIcon(":synchronize.png"));
    }else
    {
	qDebug("Sync disabled!!!");
	syncrAction->setEnabled(false);
	trayIcon->setIcon(QIcon(":red_status.png"));
    };

    connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
    connect(m_gif, SIGNAL(frameChanged(int)), this, SLOT(updateIcon()));

    IPCMedia::send(PSYNCD_IPC_NAME, PSYNCNOTIFY_IPC_NAME, PSYNCD_GET_SYNC_STATUS);
}

PSyncNotify::~PSyncNotify()
{
    m_us->finish();
    delete m_us;

/*    for(map<string,KNotification*>::iterator i=m_notificationMap.begin();i!=m_notificationMap.end();++i){
		i->second->deref();
		i->second->close();
		i->second=NULL;
	}
    m_notificationMap.clear();*/
}

void PSyncNotify::createActions()
{
    QLabel *label_nameprogram = createLabel(this, _SERVICE_NAME, Qt::AlignCenter, 2);
    label_nameprogram->setFont(createFont("Arial", 12, true, false));

    nameprogramAction = new QWidgetAction(this);
    nameprogramAction->setDefaultWidget(label_nameprogram);


    QLabel *label_lastsynctime = createLabel(this, i18n("      The last synchronization:      "), Qt::AlignCenter, 5);
    label_lastsynctime->setFont(createFont("Arial", 10, false, true));

    lastsynctimeAction = new QWidgetAction(this);
    lastsynctimeAction->setDefaultWidget(label_lastsynctime);

    syncrAction = new QAction(i18n("Syncronize"), this);
    syncrAction->setEnabled(false);
    connect(syncrAction, SIGNAL(activated()), this, SLOT(syncronizeStartStop()) );

    opensettingsAction = new QAction(i18n("Settings..."), this);
    connect(opensettingsAction, SIGNAL(activated()), this, SLOT(openSettings()));
}

void PSyncNotify::closeApp()
{
    m_app->setQuitOnLastWindowClosed(true);
    close();
}

void PSyncNotify::createTrayIcon()
{
    trayIconMenu = new QMenu(this);
    trayIconMenu->addAction(nameprogramAction);
    trayIconMenu->addSeparator();
    trayIconMenu->addAction(lastsynctimeAction);
    trayIconMenu->addAction(syncrAction);
    trayIconMenu->addAction(opensettingsAction);

    trayIcon = new QSystemTrayIcon(this);
    trayIcon->setContextMenu(trayIconMenu);
    trayIcon->setIcon(*m_icon);
    trayIcon->setToolTip(_PROGRAMM_NAME);

    m_na.attachToMenu(trayIconMenu,lastsynctimeAction);

    if(m_show_tray_icon) trayIcon->show();
}

QLabel* PSyncNotify::createLabel(QWidget *widget, const QString &text, Qt::AlignmentFlag aligment, int margin)
{
    QLabel *label = new QLabel(widget);
    label->setText(text);
    label->setAlignment(aligment);
    label->setMargin(margin);
    return label;
}

QFont PSyncNotify::createFont(const QString &name, int size, bool is_bold, bool is_italic)
{
    QFont font(name, size);
    font.setBold(is_bold);
    font.setItalic(is_italic);

    return font;
}

void PSyncNotify::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
	int x_pos;
	trayIconMenu->adjustSize();
	QRect d = QApplication::desktop()->screenGeometry();
	//qDebug("Desktop x=%d,y=%d,h=%d,w=%d",d.x(),d.y(),d.height(),d.width());
	QRect m = trayIconMenu->geometry();
	//qDebug("Menu x=%d,y=%d,h=%d,w=%d",r.x(),r.y(),r.height(),r.width());
	QRect i = trayIcon->geometry();
	//qDebug("Icon x=%d,y=%d,h=%d,w=%d",i.x(),i.y(),i.height(),i.width());
	x_pos = d.width()-m.width();
	if(x_pos>i.x()) x_pos=i.x();
	trayIconMenu->move(x_pos,i.y()-m.height());
	trayIconMenu->show();
}

void PSyncNotify::updateMenuPosAndSize()
{
	int x_pos;
	trayIconMenu->adjustSize();
	QRect d = QApplication::desktop()->screenGeometry();
	//qDebug("Desktop x=%d,y=%d,h=%d,w=%d",d.x(),d.y(),d.height(),d.width());
	QRect m = trayIconMenu->geometry();
	//qDebug("Menu x=%d,y=%d,h=%d,w=%d",r.x(),r.y(),r.height(),r.width());
	QRect i = trayIcon->geometry();
	//qDebug("Icon x=%d,y=%d,h=%d,w=%d",i.x(),i.y(),i.height(),i.width());
	x_pos = d.width()-m.width();
	if(x_pos>i.x()) x_pos=i.x();
	trayIconMenu->move(x_pos,i.y()-m.height());
}



void PSyncNotify::Notify_Close()
{
/*	string name;
	KNotification* src= qobject_cast<KNotification*>(sender());
	for(map<QString,KNotification*>::iterator i=m_notificationMap.begin();i!=m_notificationMap.end();++i)
		if(i->second == src){
			i->second=NULL;
		}
*/
}
void PSyncNotify::receiveNotification()
{
	QString msg;
    IPCPacket *m_pkt = NULL;
    int total_bytes = m_us->bytesAvailable();
    bool is_receiving_file =false;
    if (m_pkt)
        return;

    m_pkt = new IPCPacket();
    m_us->readDatagram((char*)m_pkt->readBuffer(total_bytes), total_bytes);
    QString filename;
    int persent;

    // TODO: Use 'if (!m_pkt->IsValid()) return;' Luke!!
    if (m_pkt->IsValid())
    {
    	//qDebug("IPCPacket received: %lu datalen: %lu",m_pkt->getType(),m_pkt->getDataLen());
        switch (m_pkt->getType())
        {
            case PSYNCNOTIFY_NOTIFICATION_MESSAGE:
                KNotification::event(KNotification::Notification, _NOTIFICATION_CAPTION, QString::fromLocal8Bit((const char*)m_pkt->getData(), m_pkt->getDataLen()), trayIcon->icon().pixmap(64,64), trayIcon->contextMenu());
                break;
	    case PSYNCNOTIFY_FILERECV_PROGRESS:
	    	is_receiving_file=true;
	    case PSYNCNOTIFY_FILESEND_PROGRESS:
			if(m_pkt->getDataLen()<=sizeof(int)) break;
			persent=*((int*)(m_pkt->getData()));
			filename = QString::fromLocal8Bit(QByteArray((const char*)m_pkt->getData()+sizeof(int),m_pkt->getDataLen()-sizeof(int)));
			//qDebug("PSYNCNOTIFY_FILE*_PROGRESS: %s, %d %",filename.toLocal8Bit().data(),persent);

			if(!m_na.exist(filename)) m_na.add(filename);

			if(is_receiving_file)
				m_na.setText(filename,i18n("Updating \"%1\"",filename));
			else
				m_na.setText(filename,i18n("Commiting \"%1\"",filename));

			m_na.setPercent(filename,persent);
			updateMenuPosAndSize();
			break;
			
	    case PSYNCNOTIFY_HTTP_ERROR:
			filename=QString::fromLocal8Bit(QByteArray((const char*)m_pkt->getData(),m_pkt->getDataLen()));
			msg = i18n("Server interaction error (%1)!",filename);
			KNotification::event(KNotification::Notification, _NOTIFICATION_CAPTION, msg, trayIcon->icon().pixmap(64,64), trayIcon->contextMenu());
			break;
	    case PSYNCNOTIFY_FILERECV_ERROR:
	    case PSYNCNOTIFY_FILESEND_ERROR:
			filename=QString::fromLocal8Bit(QByteArray((const char*)m_pkt->getData(),m_pkt->getDataLen()));
			if(is_receiving_file)
				msg = i18n("Error updating \"%1\"!",filename);
			else
				msg = i18n("Error commiting \"%1\"!",filename);
			KNotification::event(KNotification::Notification, _NOTIFICATION_CAPTION, msg, trayIcon->icon().pixmap(64,64), trayIcon->contextMenu());
			m_na.remove(filename);
			updateMenuPosAndSize();
			break;
	    case PSYNCNOTIFY_FILERECV_NOSPACE:
			filename=QString::fromLocal8Bit(QByteArray((const char*)m_pkt->getData(),m_pkt->getDataLen()));
			KNotification::event(KNotification::Notification, _NOTIFICATION_CAPTION, i18n("Error updating \"%1\": not enough disk space!",filename), trayIcon->icon().pixmap(64,64), trayIcon->contextMenu());
			m_na.remove(filename);
			updateMenuPosAndSize();
			break;
	    case PSYNCNOTIFY_FILERECV_COMPLETED:
	    	is_receiving_file=true;
	    case PSYNCNOTIFY_FILESEND_COMPLETED:
			filename=QString::fromLocal8Bit(QByteArray((const char*)m_pkt->getData(),m_pkt->getDataLen()));
			if(is_receiving_file)
				msg =  i18n("File \"%1\" successfully updated",filename);
			else
				msg =  i18n("File \"%1\" successfully commited!",filename);
			KNotification::event(KNotification::Notification, _NOTIFICATION_CAPTION, msg, trayIcon->icon().pixmap(64,64), trayIcon->contextMenu());
			m_na.remove(filename);
			updateMenuPosAndSize();
			break;
	    case PSYNCD_ANS_SYNC_STATUS_ONLINE:
	    case PSYNCNOTIFY_SYNC_START:
			m_sync_is_running = true;
			syncrAction->setText(i18n("Stop synchronization"));
			syncrAction->setEnabled(true);
			((QLabel*)lastsynctimeAction->defaultWidget())->setVisible(false);
			lastsynctimeAction->setVisible(false);
			updateMenuPosAndSize();
			m_gif->start();
			reportSyncStart();
			break;
	    case PSYNCNOTIFY_SYNC_STOP:
			m_sync_is_running = false;
			m_gif->stop();
			syncrAction->setText(i18n("Synchronize"));
			syncrAction->setEnabled(true);
			QTimer::singleShot(500,this,SLOT(updateLastSyncTimeLabel()));
			trayIcon->setIcon(QIcon(":synchronize.png"));
			lastsynctimeAction->setVisible(true);
			((QLabel*)lastsynctimeAction->defaultWidget())->setVisible(true);
			updateMenuPosAndSize();
			reportSyncStop();
			// update last sync
			break;
	    case PSYNCNOTIFY_SYNC_ENABLED:
			syncrAction->setEnabled(true);
			trayIcon->setIcon(QIcon(":synchronize.png"));
			break;
	    case PSYNCNOTIFY_SYNC_DISABLED:
			syncrAction->setEnabled(false);
			trayIcon->setIcon(QIcon(":red_status.png"));
			break;
		case PSYNCNOTIFY_SHOW_TRAY_ICON:
        	trayIcon->show();
        	m_show_tray_icon = true;
			break;
		case PSYNCNOTIFY_HIDE_TRAY_ICON:
        	trayIcon->hide();
        	m_show_tray_icon = false;
			break;
		case PSYNCD_ANS_SYNC_STATUS_OFFLINE:
			break;
		default:
                qDebug("Unknown type of message. The number of message is %d", (int)m_pkt->getType());
        }
    }

    if (m_pkt)
        delete m_pkt;
}

// Tray button slot
void PSyncNotify::syncronizeStartStop()
{
    IPCPacket* pkt = NULL;

    if (m_sync_is_running)
    {
        pkt = IPCPacket::Create(PSYNCD_SYNC_STOP, (char*)PSYNCNOTIFY_IPC_NAME,0,0);
    }
    else
    {
    	pkt = IPCPacket::Create(PSYNCD_SYNC_START, (char*)PSYNCNOTIFY_IPC_NAME,0,0);
    }

    if(m_us->writeDatagram((const char*)pkt->getBuffer(), pkt->getSize(), PSYNCD_IPC_NAME)>=0)
    {
    	syncrAction->setEnabled(false);
    }

    if (pkt) delete pkt;
}

void PSyncNotify::initSocket()
{
    m_us = new UnixSocket();
    if (m_us->init())
    {

        QString self_address(PSYNCNOTIFY_IPC_NAME);
        qDebug("sockfd = %d", m_us->getSockfd());
        QFileInfo fi(PSYNCNOTIFY_IPC_NAME);
        if (!fi.absoluteDir().exists())
        	fi.absoluteDir().mkdir(PSYNCNOTIFY_IPC_NAME);

        if (m_us->bind(self_address.toLocal8Bit().data()))
        {
            m_sn = new QSocketNotifier(m_us->getSockfd(), QSocketNotifier::Read);
            connect(m_sn , SIGNAL(activated(int )), this, SLOT(receiveNotification()));
        }
        else
        {
            qDebug("Error of local socket bind");
            QMessageBox::critical(this, _PROGRAMM_NAME, i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
        }
    }
    else
    {
        qDebug("Error of local socket init");
        QMessageBox::critical(this, _PROGRAMM_NAME, i18n("The program error occurred. Information about the process and since the last synchronization may be displayed incorrectly"), QMessageBox::Ok);
    }
}

void PSyncNotify::openSettings()
{
	m_na.remove("test2");
    KProcess* process = new KProcess();
    //*process << QString(GLOBAL_EXECUTABLE_NAME_CONFIG);
    process->setProgram(QString("kcmshell4"), QStringList("kcm_sync"));
    process->start();
}

void PSyncNotify::readSettings()
{
    int size_value=512;
    char value[size_value];

    //get_param_str(m_pathname_file_config.toLocal8Bit().data(), SETTINGS_SHOW_TRAY, value, size_value);
    get_param_str(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_SHOW_TRAY, value, size_value);
    m_show_tray_icon = QVariant(value).toInt();

    get_param_str(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_SYNC_IS, value, size_value);
    m_sync_on = (Qt::CheckState)QVariant(value).toInt();
}

void PSyncNotify::updateLastSyncTimeLabel()
{
    unsigned int time=0;
    QString last_sync_time= i18n("Unknown");

    get_param_ui(GLOBAL_SERVICE_CONFIG_DIR, SETTINGS_LAST_SYNC_TIME, &time);
    if(time)
    {
    	QDateTime dt;
    	dt.setTime_t(time);
    	KLocale *aLocale = KGlobal::locale();
    	last_sync_time = aLocale->formatDateTime(dt, KLocale::FancyShortDate).toLower();
    };

    setLastSyncTimeLabel(i18n("      The last synchronization: ") + last_sync_time);
}

void PSyncNotify::setLastSyncTimeLabel(QString str)
{
	QLabel *label = (QLabel*)lastsynctimeAction->defaultWidget();
	label->setText(str);
}


void PSyncNotify::updateIcon()
{
    trayIcon->setIcon(m_gif->currentPixmap());
}

void PSyncNotify::reportSyncStart()
{
	QString time_string = KGlobal::locale()->formatDateTime(QDateTime::currentDateTime(), KLocale::FancyShortDate);
	KNotification::event(KNotification::Notification, _NOTIFICATION_CAPTION, i18n("%1 : Synchronization start! ",time_string), trayIcon->icon().pixmap(64,64), trayIcon->contextMenu(),KNotification::CloseOnTimeout);
}

void PSyncNotify::reportSyncStop()
{
	QString time_string = KGlobal::locale()->formatDateTime(QDateTime::currentDateTime(), KLocale::FancyShortDate);
	KNotification::event(KNotification::Notification, _NOTIFICATION_CAPTION, i18n("%1 : Synchronization end!",time_string), trayIcon->icon().pixmap(64,64), trayIcon->contextMenu(),KNotification::CloseOnTimeout);
}
