#define HTTP_PORT 	443
#include <string>
#include <fstream>
#include <sstream>
#include <cstdlib>
#include <errno.h>
#include <stdio.h>
#include <memory.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <map>
#include <libcfg.h>
#include <dirent.h>
#include "WebDAV.h"
#include "http_hdr.h"
#include "tinyxml.h"

#define NOLOCK

static void NamespaceClear(TiXmlNode* doc)
{
	if(!doc)
		return;
	string tmp;
	TiXmlNode* pChild;
	tmp=doc->ValueStr();
	if(tmp.find(":")!=string::npos){
		doc->SetValue(tmp.substr(tmp.find(":")+1,string::npos));
	}
	for ( pChild = doc->FirstChild(); pChild != 0; pChild = pChild->NextSibling())
		NamespaceClear(pChild);

}
//static 
using namespace std;

WebDav::WebDav()
{
	pthread_spin_init(&tlock,0);
}
     

WebDav::~WebDav()
{
	pthread_spin_destroy(&tlock);
}
//Приватные члены)
//Файлы
string WebDav::getFileSTag(string uname)
{
	return propfind(uname,string(),"getstag");
}

int WebDav::Lock()
{
	int ret=0;
	pthread_spin_lock(&tlock);
#ifndef NOLOCK
	ret=lock(srv, lock_token);
	if(ret){
		cerr<<"LOCK error:"<<ret<<endl;
		return -1;
	}
#endif
	return ret;
}

int WebDav::Unlock()
{
	int ret=0;
#ifndef NOLOCK
	ret=unlock(srv, lock_token);
	if(ret){
		cerr<<"UNLOCK error:"<<ret<<endl;
	}
#endif
	pthread_spin_unlock(&tlock);
	lock_token.clear();
	return ret;
}

int WebDav::Delete(const string path)
{
	string p1=path;
	WebDav::_delete(path);
	while(true){
		if(p1.rfind("/")==string::npos)
			break;
		p1=p1.substr(0,p1.rfind("/"));	
		if(!(GetServerList(p1+"/").empty()))	//Если папка не пустая
			break;
		if(GetServerState(p1+"/").valid())		//Если папка в синхронизации
			break;
		WebDav::_delete(p1+"/");
	}
	return 0;
}


void WebDav::SetMeta(string uname,map<string, string> meta)
{
	setprop(uname, meta, lock_token);
}
/*int WebDav::SetMode(string uname,mode_t mode)
{
	int ret;
	ostringstream s;
	s<<mode;
	ret=setprop(uname, "filemode", s.str(), lock_token);
	return ret;
}

int WebDav::SetSTag(string uname,string sTag)
{
	int ret;
	ret=setprop(uname, "getstag", sTag, lock_token);
	return ret;
}
*/
int WebDav::GetMode(string uname,mode_t* mode)
{
	string ret=propfind(uname, lock_token,"filemode");
	stringstream s(ret);
	s>>*mode;
	return last_error;
}

int WebDav::GetSTag(string uname,string* sTag)
{
	*sTag = propfind(uname, lock_token,"getstag");
	return last_error;
}

int WebDav::GetCL(string uname,unsigned* cl)
{
	string sCL = propfind(uname, lock_token,"getcontentlength");
	istringstream s(sCL);
	s>>*cl;
	return last_error;
}


int WebDav::Mkcol(const string path)
{
	try{
		mkcol(path, lock_token);
	}catch(http_exception::conflict){
		Create_path(path);
		return 0;
	}catch(http_exception::not_allowed){		//Каталог уже есть - пропускаем
	}catch(http_exception::error e){
		cerr<<"ошибка создания файла на сервере:"<<e.status()<<endl;
		throw;
	}
	return 0;
}

int WebDav::Create_path(const string uname)
{
	string path=uname.substr(0,uname.rfind("/")+1);
	string::size_type i=0;
	while(true){
		i=path.find("/",i+1);
		if(i == string::npos)
			break;
		string p1=path.substr(0,i);
		try{
			Mkcol(p1);
		}catch(http_exception::moved){
		}
	}
	return 0;
}

int WebDav::Put(string uname, string path)
{
	try{
		try_put(uname, lock_token);
		put(path, uname, lock_token);
	}catch(http_exception::forbidden){
		Create_path(uname);
		put(path, uname, lock_token);
		return 0;
	}catch(http_exception::error e){
		cerr<<"ошибка создания файла на сервере:"<<e.status()<<endl;
		throw;
	}
//	setprop(uname, "clientpath", path, lock_token);
	return 0;
}

int WebDav::Get(string uname, string path)
{
	int ret=get(path, uname, lock_token);
	return ret;
}
sync_object WebDav::GetServerState(string uname)
{
	string _uname;
	string path;			
	mode_t mode;
	string sTag;
	char level;
	unsigned cl;

	string allprop=propfind(uname, lock_token ,string());
	//XML
	TiXmlDocument ret_request;
	ret_request.Parse(allprop.c_str());
	NamespaceClear(ret_request.RootElement());
	//TO multistatus
	TiXmlNode* parent=ret_request.FirstChild("multistatus");
	while(parent){
		TiXmlNode* child = parent->FirstChild("response");
		TiXmlHandle i(child);
		TiXmlElement* element;
		//Путь к файлу на сервере
		element = i.FirstChild("href").ToElement();
		if(!element)
			break;
		_uname = element->GetText();
		_uname = _uname.substr(conn_ctx.path.length());				//Обрезаем до имени файла	
		_uname = http_hdr::from_persent_byte(_uname);
		if(uname!=_uname)
			break;
		//Путь на клиенте
		element = i.FirstChild("propstat").FirstChild("prop").FirstChild("clientpath").ToElement();
		if(!element)
			path=string();
		else
			path = element->GetText();
		//Етаг
		element = i.FirstChild("propstat").FirstChild("prop").FirstChild("getstag").ToElement();
		if(!element)
			sTag=string();
		else
			sTag = element->GetText();
		//Mode
		element = i.FirstChild("propstat").FirstChild("prop").FirstChild("filemode").ToElement();
		if(!element)
			mode=0;
		else{
			istringstream s(element->GetText());
			s>>mode;
		}
		//level
		element = i.FirstChild("propstat").FirstChild("prop").FirstChild("level").ToElement();
		if(!element)
			level=-1;
		else{
			istringstream s(element->GetText());
			s>>level;
		}
		//CL
		element = i.FirstChild("propstat").FirstChild("prop").FirstChild("getcontentlength").ToElement();
		if(!element)
			cl=0;
		else{
			istringstream s(element->GetText());
			s>>cl;
		}
		break;
	}
	sync_object so(_uname, path, 0, sTag, mode, -1, 0,cl);
	if(level == 1)
		so.leveldown();
	return so;
}
vector<sync_object> WebDav::GetServerList(string uname)
{
	vector<sync_object> ret;
	string allprop=propfind(uname, lock_token ,string());
	//XML
	TiXmlDocument ret_request;
	ret_request.Parse(allprop.c_str());
	NamespaceClear(ret_request.RootElement());
	//TO multistatus
	TiXmlNode* parent=ret_request.FirstChild("multistatus");
	if(parent){
		TiXmlNode* child=0;	
		while((child = parent->IterateChildren("response",child))!=NULL){			//TODO:Возможно контролировать по response/propstat/status?
			string _uname;
			string path;
			mode_t mode;
			string sTag;
			char level;
			unsigned cl;
			TiXmlHandle i(child);
			TiXmlElement* element;
			//Путь к файлу на сервере
			element = i.FirstChild("href").ToElement();
			if(!element)
				continue;
			_uname = element->GetText();
			_uname = _uname.substr(conn_ctx.path.length());				//Обрезаем до имени файла	
			_uname = http_hdr::from_persent_byte(_uname);
			if(uname==_uname)
				continue;
			//Путь на клиенте
			element = i.FirstChild("propstat").FirstChild("prop").FirstChild("clientpath").ToElement();
			if(!element)
				path=string();
			else
				path = element->GetText();
			//Етаг
			element = i.FirstChild("propstat").FirstChild("prop").FirstChild("getstag").ToElement();
			if(!element)
				sTag=string();
			else
				sTag = element->GetText();
			//Mode
			element = i.FirstChild("propstat").FirstChild("prop").FirstChild("filemode").ToElement();
			if(!element)
				mode=0;
			else{
				istringstream s(element->GetText());
				s>>mode;
			}
			//level
			element = i.FirstChild("propstat").FirstChild("prop").FirstChild("level").ToElement();
			if(!element)
				level=-1;
			else{
				istringstream s(element->GetText());
				s>>level;
			}
			//CL
			element = i.FirstChild("propstat").FirstChild("prop").FirstChild("getcontentlength").ToElement();
			if(!element)
				cl=0;
			else{
				istringstream s(element->GetText());
				s>>cl;
			}


			sync_object so(_uname, path, 0, sTag, mode, -1, 0,cl);
			if(level == 1)
				so.leveldown();
			ret.push_back(so);
		}
	}
	return 	ret;
}

int WebDav::TryAuth()
{
	long long max, cur;
	if(!GetQuote(max, cur)){
		set_param_ull((conn_ctx.home+"info/").c_str(),"quota_total",max);
		set_param_ull((conn_ctx.home+"info/").c_str(),"quota_used",cur);
	}
	return 0;
}

int WebDav::GetQuote(long long& max, long long& cur)
{
	string allprop=propfind(string(),string(),string());

	//XML
	TiXmlDocument ret_request;
	ret_request.Parse(allprop.c_str());
	NamespaceClear(ret_request.RootElement());
	TiXmlHandle handle(&ret_request);
	TiXmlElement* child=handle.FirstChild("multistatus").FirstChild("response").FirstChild("propstat").FirstChild("prop").FirstChild("quota-available-bytes").ToElement();
	if(!child)
		return last_error;
	max =atoll(child->GetText());
	child=handle.FirstChild("multistatus").FirstChild("response").FirstChild("propstat").FirstChild("prop").FirstChild("quota-used-bytes").ToElement();
	if(!child)
		return last_error;
	cur =atoll(child->GetText());
	return 0;
}
