#include <sys/stat.h>
#include <errno.h>
#define NE_LFS
#include <neon/ne_socket.h>
#include <neon/ne_request.h>
#include <neon/ne_xml.h>
#include <iomanip>
#include "base64.h"
#include "main.h"
#include "http_hdr.h"
#include "WebDav_neon.h"
#include "tinyxml.h"

#define NOLOCK
static int ssl_checker(void* userdata, int failures, const ne_ssl_certificate* cert)
{
	const char *hostname = (const char*)userdata;
//TODO:	dump_cert(cert);
	
	cerr<<"Certificate verification failed - the connection may have been intercepted by a third party!"<<endl;
	
	if (failures & NE_SSL_IDMISMATCH) { 
		const char *id = ne_ssl_cert_identity(cert);
		if (id) 
			cerr<<"Server certificate was issued to "<<id<<" not "<<hostname<<"."<<endl;
		else
			cerr<<"The certificate was not issued for" <<hostname<<endl;
	}

	if (failures & NE_SSL_UNTRUSTED)
		cerr<<"The certificate is not signed by a trusted Certificate Authority."<<endl;
//	return 1; /* fail verification */
	return 0; /* trust the certificate anyway */
}
static string to_persent_byte(string in)
{
	ostringstream ret;
	for(string::iterator i =in.begin(); i!=in.end(); ++i){
		if(isalnum(*i) || *i=='/')
			ret<<*i;
		else
			ret<<"%"<< hex << setfill('0') << setw(2)<<((unsigned int)((unsigned char)*i));
	}
	return ret.str();
}

string from_persent_byte(string in)
{
	ostringstream ret;
	for(string::iterator i =in.begin(); i!=in.end(); ++i){
		if(*i!='%')
			ret<<*i;
		else{
			++i;
			string str(i,i+2);
			ret<<(unsigned char)strtol(str.c_str(),0,16);
			++i;
		}
	}
	return ret.str();
}


static void NamespaceClear(TiXmlNode* doc)
{
	if(!doc)
		return;
	string tmp;
	TiXmlNode* pChild;
	tmp=doc->ValueStr();
	if(tmp.find(":")!=string::npos){
		doc->SetValue(tmp.substr(tmp.find(":")+1,string::npos));
	}
	for ( pChild = doc->FirstChild(); pChild != 0; pChild = pChild->NextSibling())
		NamespaceClear(pChild);

}
WebDav_neon::WebDav_neon()
{
	if(ne_sock_init())
		throw "NEON initialization failure";
	sess=ne_session_create("https", conn_ctx.host.c_str(), conn_ctx.port);
	ne_ssl_set_verify(sess, ssl_checker, (void*)conn_ctx.host.c_str());
}
     

WebDav_neon::~WebDav_neon()
{
	ne_session_destroy(sess);
	ne_sock_exit();
}
//Команды Вебдав
/** Получение одного свойства файла
 *  если нет lock_token - не использовать блокировку, 
 *  если нет prop - отдать просто все свойства в результате
 *  \param r_path - путь для получения свойств
 * 
 */
string WebDav_neon::propfind(string r_path, string lock_token, string prop)
{
	//XML
	TiXmlDocument request;
	TiXmlDeclaration* decl=new TiXmlDeclaration("1.0","utf-8","");
	TiXmlElement* lp= new TiXmlElement("propfind");
	TiXmlElement* la= new TiXmlElement("allprop");
	lp->SetAttribute("xmlns","DAV:");
	lp->LinkEndChild(la);
	request.LinkEndChild(decl);
	request.LinkEndChild(lp);

	TiXmlPrinter print;
	print.SetIndent(" ");
	request.Accept(&print);
	//\XML
/*	http_hdr hdr(http_hdr::CM_PFIND, r_path);
	hdr.values["Depth"]="1";
	hdr.add_lt(lock_token);
	hdr.add_body(print.Str().c_str(), print.Str().size());
	try{
		hdr.process_request();
	}catch(http_exception::not_found){
		return string();
	}*/
	ne_request *req = ne_request_create(sess, "PROPFIND", (conn_ctx.path+to_persent_byte(r_path)).c_str());
	vector<char> rdata;
	try{
		ne_add_request_header(req, "Depth", "1");
		ne_add_request_header(req, "Authorization", (string("Basic ")+base64_encode(conn_ctx.creds)).c_str());
		if(!lock_token.empty())
			ne_add_request_header(req, "If",(string("(")+lock_token+")").c_str());
		ne_set_request_body_buffer(req, print.Str().c_str(), print.Str().size());
		ne_add_request_header(req, "Content-Type", NE_XML_MEDIA_TYPE);
		if(ne_begin_request (req))
			   throw string("Request failed: ")+ ne_get_error(sess);
		char buf[1024];
		int cnt;
		while((cnt = ne_read_response_block(req, buf, 1024)) > 0){
			rdata.insert(rdata.end(),buf,buf+cnt);		
		}
		if(cnt<0){
			ne_end_request(req);
			throw string("Request failed: ")+ ne_get_error(sess);
		}

		ne_end_request(req);
		http_exception::error::check(ne_get_status(req)->code);
	}catch(http_exception::not_found){
		ne_request_destroy(req);
		return string();
	}catch(http_exception::moved){					// Меняем сервер
		conn_ctx.host=ne_get_response_header(req,"Location");
		string::size_type i=conn_ctx.host.find("//")+2;
		conn_ctx.host=conn_ctx.host.substr(i, conn_ctx.host.find("/",i)-i);
		ne_request_destroy(req);
		ne_session_destroy(sess);
		sess=ne_session_create("https", conn_ctx.host.c_str(), conn_ctx.port);
		ne_ssl_set_verify(sess, ssl_checker, (void*)conn_ctx.host.c_str());
		throw;
	}catch(...){
		ne_request_destroy(req);
		throw;
	}
	ne_request_destroy(req);

	if(prop.empty())
		return string(rdata.begin(),rdata.end());
	//Ищем свойство	
	TiXmlDocument ret_request;
	ret_request.Parse(string(rdata.begin(),rdata.end()).c_str());
	NamespaceClear(ret_request.RootElement());
	TiXmlHandle handle(&ret_request);
	TiXmlNode* child=handle.FirstChild("multistatus").FirstChild("response").FirstChild("propstat").FirstChild("prop").FirstChild(prop).FirstChild().ToNode();
	if(!child)
		return string();
	return child->ValueStr();
}


int WebDav_neon::setprop(string r_path, map<string, string>& meta, string lock_token)
{
	//XML
	TiXmlDocument request;
	TiXmlDeclaration* decl=new TiXmlDeclaration("1.0","utf-8","");
	TiXmlElement* lp= new TiXmlElement("propertyupdate");
	lp->SetAttribute("xmlns","DAV:");
	TiXmlElement* ls= new TiXmlElement("set");
	TiXmlElement* lp1= new TiXmlElement("prop");
	ls->LinkEndChild(lp1);
	request.LinkEndChild(decl);
	request.LinkEndChild(lp);
	lp->LinkEndChild(ls);
		//META
	for(map<string,string>::const_iterator i=meta.begin(); i!= meta.end(); i++){
		TiXmlElement* metaname= new TiXmlElement(i->first);
		TiXmlText* metaval= new TiXmlText(i->second);
		metaname->LinkEndChild(metaval);
		lp1->LinkEndChild(metaname);
	}
		//\META


	TiXmlPrinter print;
	print.SetIndent(" ");
	request.Accept(&print);
	//\XML
	
/*	http_hdr hdr(http_hdr::CM_PPATCH, r_path);
	hdr.add_lt(lock_token);
	hdr.add_body(print.Str().c_str(), print.Str().size());

	hdr.process_request();*/
	ne_request *req = ne_request_create(sess, "PROPPATCH", (conn_ctx.path+to_persent_byte(r_path)).c_str());
	try{
		ne_add_request_header(req, "Authorization", (string("Basic ")+base64_encode(conn_ctx.creds)).c_str());
		if(!lock_token.empty())
			ne_add_request_header(req, "If",(string("(")+lock_token+")").c_str());
		ne_set_request_body_buffer(req, print.Str().c_str(), print.Str().size());
		ne_add_request_header(req, "Content-Type", NE_XML_MEDIA_TYPE);
		if (ne_request_dispatch(req))
			   throw string("Request failed: ")+ne_get_error(sess);
		http_exception::error::check(ne_get_status(req)->code);
	}catch(...){
		ne_request_destroy(req);
		throw;
	}
	ne_request_destroy(req);
	return 0;
}

int WebDav_neon::try_put(string r_path, string lockt)
{
/*
	http_hdr hdr(http_hdr::CM_PUT, r_path);
	hdr.add_lt(lockt);
	hdr.add_body("~", 1);
	
	hdr.process_request();
*/
	ne_request *req = ne_request_create(sess, "PUT", (conn_ctx.path+to_persent_byte(r_path)).c_str());
	ne_add_request_header(req, "Authorization", (string("Basic ")+base64_encode(conn_ctx.creds)).c_str());
	try{
		if(!lock_token.empty())
			ne_add_request_header(req, "If",(string("(")+lock_token+")").c_str());
		ne_set_request_body_buffer(req, "~", 1);
		if (ne_request_dispatch(req))
			   throw string("Request failed: ")+ne_get_error(sess);
		http_exception::error::check(ne_get_status(req)->code);
	}catch(...){
		ne_request_destroy(req);
		throw;
	}
	ne_request_destroy(req);

	return 0;
}
static string pfilename;
static int _pt=-1; // antispam Percent counter
static ssize_t  fd_to_body(void *userdata, char *buffer, size_t buflen)
{
	std::ifstream* fd=(std::ifstream*)userdata;
	if(!buflen ){
		fd->seekg(0, ios_base::beg);
		return 0;
	}
	if(conn_ctx.stopflag)
		return -1;
	if(fd->eof())
		return 0;
	fd->read(buffer,buflen);

	if(fd->eof()){
		return fd->gcount();
	}

	unsigned long long transmitted=fd->tellg();
	fd->seekg(0,ios_base::end);
	unsigned mcl=fd->tellg();
	fd->seekg(transmitted, ios::beg);
	ostringstream msg;
	int pt= (transmitted*100)/mcl;
	msg.write((char*)&pt,sizeof(int));
	msg<<pfilename;
	if(pt!=_pt){
		conn_ctx.ipc->send(PSYNCNOTIFY_FILESEND_PROGRESS, PSYNCNOTIFY_IPC_NAME, msg.str());
		_pt=pt;
	}

	if(fd->fail())
		return -1;
		
	return fd->gcount();
}

int WebDav_neon::put(string filename,string r_path, string lockt)
{
	struct stat st;
	std::ifstream* fd=new std::ifstream(filename.c_str());	

//	fd.exceptions(ios_base::badbit|ios_base::eofbit|ios_base::failbit);
	if ( !fd->is_open() ) 
		return -ENOENT;

	if(stat(filename.c_str(), &st)){	
		cerr<< "File "<<filename.c_str()<<" stat error"<<endl;
		return errno;
	}
	pfilename=filename;
/*
//	ostringstream msg;
//	msg<<"Загружаем на сервер "<< r_path;
//	conn_ctx.ipc->send(ipc_proxy::NOTE_MSG, conn_ctx.pathNotify.c_str(), msg.str().c_str());

	http_hdr hdr(http_hdr::CM_PUT, r_path);
	hdr.add_lt(lockt);
	hdr.add_file_body(&fd, st.st_size);

	hdr.process_request();
*/
	ne_request *req = ne_request_create(sess, "PUT", (conn_ctx.path+to_persent_byte(r_path)).c_str());
	ne_add_request_header(req, "Authorization", (string("Basic ")+base64_encode(conn_ctx.creds)).c_str());
	try{
		if(!lock_token.empty())
			ne_add_request_header(req, "If",(string("(")+lock_token+")").c_str());
		ne_set_request_body_provider(req, (ne_off_t)st.st_size, fd_to_body, (void*)fd);

		if (ne_request_dispatch(req)){
			if(!conn_ctx.stopflag)
				throw string("Request failed: ")+ne_get_error(sess);
			else
				throw "Sync emergensy stop";
		}
		http_exception::error::check(ne_get_status(req)->code);
		conn_ctx.ipc->send(PSYNCNOTIFY_FILESEND_COMPLETED, PSYNCNOTIFY_IPC_NAME, pfilename);
	}catch(...){
		ne_request_destroy(req);
		delete fd;
		throw;
	}
	ne_request_destroy(req);
	delete fd;

	return 0;
}
static int _rpt=-1;	//ANTISPAM
int WebDav_neon::get(string filename, string r_path, string lockt)
{
	ofstream fd(filename.c_str(), ios_base::trunc|ios_base::binary);	

	if ( !fd.is_open() ) {
		cerr<<"Ошибка записи файла "<< filename<<" Сохраняем в каталог по умолчанию";
		string npath(conn_ctx.defFolder);
		npath+="/";
		npath+=filename.substr(filename.rfind("/"));
		fd.clear();
		fd.open(npath.c_str(), ios_base::trunc|ios_base::binary);

		if ( !fd.is_open() ){
			cerr<<"Ошибка записи файла "<< filename;
			return -ENOENT;
		}
	}
/*	http_hdr hdr(http_hdr::CM_GET, r_path);
	dr.add_lt(lockt);
	hdr.add_file_rbody(&fd);
	if(hdr.process_request()==http_hdr::HH_FAIL){
		fd.close();
		unlink(filename.c_str());
	}*/
	ne_request *req = ne_request_create(sess, "GET", (conn_ctx.path+to_persent_byte(r_path)).c_str());
	try{
		ne_add_request_header(req, "Authorization", (string("Basic ")+base64_encode(conn_ctx.creds)).c_str());
		if(!lock_token.empty())
			ne_add_request_header(req, "If",(string("(")+lock_token+")").c_str());
		if(ne_begin_request (req))
			   throw string("Request failed: ")+ ne_get_error(sess);
		http_exception::error::check(ne_get_status(req)->code);
		const char* str_cl=ne_get_response_header(req, "Content-Length");
		unsigned cl;
		if(str_cl)
			cl=atoi(str_cl);
		else{
			str_cl=ne_get_response_header(req, "ETag");
			if(!str_cl)
				throw "Invalid cl";
			cl=strtol(str_cl+1, NULL, 16);
		}


static		char buf[20480];
		int cnt;
		while((cnt = ne_read_response_block(req, buf, 20480)) > 0){
			fd.write(buf,cnt);
			ostringstream msg;
			long long transferred=fd.tellp();
			transferred*=100;
			int pt=(transferred/cl);
			msg.write((char*)&pt,sizeof(int));
			msg<<filename;
			if(pt!=_rpt){
				conn_ctx.ipc->send(PSYNCNOTIFY_FILERECV_PROGRESS, PSYNCNOTIFY_IPC_NAME, msg.str());
				_rpt=pt;
			}
			if(conn_ctx.stopflag){
				throw "Sync emergency stop";
			}
		}
		if(cnt<0)
			throw string("Request failed: ")+ ne_get_error(sess);
		conn_ctx.ipc->send(PSYNCNOTIFY_FILERECV_COMPLETED, PSYNCNOTIFY_IPC_NAME, filename);

		ne_end_request(req);
	}catch(http_exception::error){
		ne_end_request(req);
		ne_request_destroy(req);
		throw;
	}catch(...){
		ne_end_request(req);
		ne_request_destroy(req);
		throw;
	}
	ne_request_destroy(req);


	return 0;
}
int WebDav_neon::mkcol(string r_path, string lockt)
{

/*	http_hdr hdr(http_hdr::CM_MKCOL, r_path);
	hdr.add_lt(lockt);
	hdr.process_request();
		*/
	ne_request *req = ne_request_create(sess, "MKCOL", (conn_ctx.path+to_persent_byte(r_path)).c_str());
	try{
		ne_add_request_header(req, "Authorization", (string("Basic ")+base64_encode(conn_ctx.creds)).c_str());
		if(!lock_token.empty())
			ne_add_request_header(req, "If",(string("(")+lock_token+")").c_str());
		if (ne_request_dispatch(req))
			   throw string("Request failed: ")+ne_get_error(sess);
		http_exception::error::check(ne_get_status(req)->code);
	}catch(http_exception::error){
		ne_request_destroy(req);
		throw;
	}
	ne_request_destroy(req);

	return 0;
}

int WebDav_neon::lock(string r_path, string& lock_token)
{
	TiXmlDocument request;
	TiXmlDeclaration* decl=new TiXmlDeclaration("1.0","utf-8","");
	TiXmlElement* li= new TiXmlElement("lockinfo");
	TiXmlElement* ls= new TiXmlElement("lockscope");
	TiXmlElement* scope= new TiXmlElement("exclusive");
	TiXmlElement* lt= new TiXmlElement("locktype");
	TiXmlElement* type= new TiXmlElement("write");

	ls->LinkEndChild(scope);
	li->LinkEndChild(ls);
	li->SetAttribute("xmlns","DAV:");
	lt->LinkEndChild(type);
	li->LinkEndChild(lt);
	request.LinkEndChild(decl);
	request.LinkEndChild(li);

	TiXmlPrinter print;
	print.SetIndent(" ");
//	print.SetStreamPrinting();
	request.Accept(&print);

	ne_request *req = ne_request_create(sess, "LOCK", (conn_ctx.path+to_persent_byte(r_path)).c_str());
	try{
		ne_add_request_header(req, "Authorization", (string("Basic ")+base64_encode(conn_ctx.creds)).c_str());
		ne_set_request_body_buffer(req, print.Str().c_str(), print.Str().length());
		if (ne_request_dispatch(req))
			   throw string("Request failed: ")+ne_get_error(sess);
		http_exception::error::check(ne_get_status(req)->code);
	}catch(http_exception::error){
		ne_request_destroy(req);
		throw;
	}
	lock_token=ne_get_response_header(req, "Lock-Token");
	ne_request_destroy(req);


/*	http_hdr hdr(http_hdr::CM_LOCK, r_path);
	hdr.add_lt(lock_token);
	hdr.add_body(print.Str().c_str(), print.Str().size());
	hdr.values["Timeout"]="Second-60";

	hdr.process_request();
	TiXmlDocument ret_request;
	ret_request.Parse(hdr.rdata);
	lock_token=hdr.values["Lock-Token"].substr(0,hdr.values["Lock-Token"].length()-1);
*/
	return 0;
}

int WebDav_neon::unlock(string r_path, string lockt)
{
/*	http_hdr hdr(http_hdr::CM_UNLOCK, r_path);
	hdr.add_lt(lockt);
	hdr.process_request();
		*/
	ne_request *req = ne_request_create(sess, "UNLOCK", (conn_ctx.path+to_persent_byte(r_path)).c_str());
	try{
		ne_add_request_header(req, "Authorization", (string("Basic ")+base64_encode(conn_ctx.creds)).c_str());
		ne_add_request_header(req, "If",(string("(")+lockt+")").c_str());
		if (ne_request_dispatch(req))
			   throw string("Request failed: ")+ne_get_error(sess);
		http_exception::error::check(ne_get_status(req)->code);
	}catch(http_exception::error){
		ne_request_destroy(req);
		throw;
	}
	lock_token=string();
	ne_request_destroy(req);


	return 0;
}

int WebDav_neon::_delete(const string path)
{
/*	try{
		http_hdr hdr(http_hdr::CM_DELETE, path);
		hdr.process_request();
	}catch(http_exception::not_found){}//И ладно(ктото удалил до нас)*/
	ne_request *req = ne_request_create(sess, "DELETE", (conn_ctx.path+to_persent_byte(path)).c_str());
	try{
		ne_add_request_header(req, "Authorization", (string("Basic ")+base64_encode(conn_ctx.creds)).c_str());
		if (ne_request_dispatch(req))
			   throw string("Request failed: ")+ne_get_error(sess);
		http_exception::error::check(ne_get_status(req)->code);
	}catch(http_exception::not_found){//И ладно(ктото удалил до нас)*/
	}catch(http_exception::error){
		ne_request_destroy(req);
		throw;
	}
	ne_request_destroy(req);

	return 0;
}
