#include <fstream>
#include <sstream>
#include <algorithm>
#include <errno.h>
#include <time.h>
#include <stdlib.h>
#include <pthread.h>
#include "filesdb.h"
#include "uuid/uuid.h"

ostream& operator<<(ostream& s,sync_object f)
{
	string data = f.getStringData();
	unsigned sz=data.size();
	s.write((char*)&sz,sizeof(sz));
	s.write(data.c_str(),data.size());
	return s;
}

istream& operator>>(istream& s,sync_object &f)
{
	unsigned sz;
	char* buf;

	s.exceptions(ios_base::badbit|ios_base::eofbit|ios_base::failbit);
	s.read((char*)&sz,sizeof(sz));			
	buf=new char[sz];
	s.read(buf,sz);
	f=sync_object(string(buf,sz));
	delete []buf;

	return s;
}



filedb::filedb(string fpath):path(fpath)
{
	pthread_mutex_init(&m_lock,0);
}

filedb::filedb()
{
	pthread_mutex_init(&m_lock,0);
}

filedb::~filedb()
{
	pthread_mutex_destroy(&m_lock);
	for(vector<sync_object*>::iterator i=m_db.begin(); i!=m_db.end(); i++)
		delete *i;
}

void filedb::dbset(string fpath)
{
	path=fpath;
}
int filedb::lock()
{
	return pthread_mutex_lock(&m_lock);
}

int filedb::unlock()
{
	return pthread_mutex_unlock(&m_lock);
}
int filedb::load()
{
	sync_object buf;
	ifstream f(path.c_str());
	if(!f)
		return -EACCES;
	m_db.clear();
	try{
		while(f>>buf){
			buf.setDB(this);
			m_db.push_back(new sync_object(buf));
		}
	}
	catch(ios_base::failure){
	}
	f.close();
	return 0;
}

int filedb::save()
{
	ofstream f(path.c_str(),ios_base::out|ios_base::trunc);
	if(!f)
		return -EACCES;
	for(vector<sync_object*>::iterator i=m_db.begin(); i!=m_db.end(); i++)
		f<<**i;
	f.close();
	return 0;
}

sync_object* filedb::insert(sync_object obj)
{
	for(vector<sync_object*>::iterator i=m_db.begin(); i!=m_db.end(); i++)
		if(**i==obj)
			return *i;
	obj.setDB(this);
	sync_object* f= new sync_object(obj);
	m_db.push_back(f);

	return f;
}

sync_object* filedb::insert(string fpath)
{
	for(vector<sync_object*>::iterator i=m_db.begin(); i!=m_db.end(); i++)
	{
		if(**i==fpath)
		{
			if((*i)->toDelete())
			{
				lock();
				(*i)->undelete();
				save();
				unlock();
				return *i;
			}
			return *i;
		};
	};
	lock();
	sync_object* f= new sync_object(fpath,string());
	f->setDB(this);
	m_db.push_back(f);
	save();
	unlock();
	return f;
}

void filedb::delDeleted()
{
	for(vector<sync_object*>::iterator i=m_db.begin(); i!=m_db.end(); )
		if((*i)->canDelete()){
			delete *i;
			i=m_db.erase(i);
		}else 
			i++;
}

int filedb::remove(string fpath)
{
	vector<sync_object*>::iterator i;
	lock();
	for(i=m_db.begin(); i!=m_db.end(); i++)
		if(**i==fpath)
			break;
	if(i==m_db.end()){
		unlock();
		return -1;
	}
	(*i)->setDelete();
	save();
	unlock();
	return 0;
}

int filedb::erase(sync_object* obj)
{
	vector<sync_object*>::iterator i= std::find(m_db.begin(),m_db.end(),obj);
	delete *i;
	m_db.erase(i);
	return 0;
}


int filedb::next(sync_object*& iter)
{
	vector<sync_object*>::iterator i= std::find(m_db.begin(),m_db.end(),iter);
	while(i!=m_db.end()){
		++i;
		if(i!=m_db.end())
			if(!(*i)->level())
				break;
	}
	if(iter==NULL)
		i=m_db.begin();
	if(i==m_db.end())
		return 0;
	iter=*i;
	return 1;
}

int filedb::combine(vector<sync_object> srv_files)
{
	for(vector<sync_object>::iterator i=srv_files.begin();i!= srv_files.end(); i++){
		if(i->valid())
			insert(*i);
	}
	save();
	return 0;
}

sync_object* filedb::find(string fpath)
{
	for(vector<sync_object*>::iterator i=m_db.begin(); i!=m_db.end(); i++)
		if(**i==fpath)
			return *i;
	return NULL;
}

sync_object* filedb::find_wd(int wd)
{
	for(vector<sync_object*>::iterator i=m_db.begin(); i!=m_db.end(); i++)
		if((*i)->get_ifd() == wd)
			return *i;
	return NULL;
}


vector<sync_object*> filedb::findDir(string fpath)
{
	vector<sync_object*> ret;
	for(vector<sync_object*>::iterator i=m_db.begin(); i!=m_db.end(); i++)
		if((*i)->inDir(fpath))
			ret.push_back(*i);
	return ret;
}


