#include <string>
#include <iomanip>
#include <map>
#include <sstream>
#include <iostream>
#include <stdlib.h>
#include <memory.h>
#include <netdb.h>
#include <sys/socket.h>
#include <signal.h>
#include <errno.h>
#include <sys/ioctl.h>
#include "main.h"

#include "base64.h"
#include "http_hdr.h"
#include <openssl/err.h>

//#define DEBUG
#define SSL_BUFSIZE 1024*50
using namespace std;

string str_http_cmd[]=
{
	"MKCOL",
	"PUT",
	"PROPPATCH",
	"GET",
	"LOCK",
	"UNLOCK",
	"PROPFIND",
	"DELETE",
};

http_hdr::~http_hdr()
{
	if(data)
		delete[] data;
	if(rdata)
		delete[] rdata;

	if(bio_err)
		BIO_free(bio_err);

	if(conn_state!=WD_DISCONNECTED){
		//TODO: sess=SSL_get_session(ssl)
		SSL_shutdown(ssl);
		SSL_free(ssl);
		close(sock);

	}

	//SSL
	if(ssl_ctx)
		SSL_CTX_free(ssl_ctx);
}
static string to_persent_byte(string in)
{
	ostringstream ret;
	for(string::iterator i =in.begin(); i!=in.end(); ++i){
		if(isalnum(*i) || *i=='/')
			ret<<*i;
		else
			ret<<"%"<< hex << setfill('0') << setw(2)<<((unsigned int)((unsigned char)*i));
	}
	return ret.str();
}

string http_hdr::from_persent_byte(string in)
{
	ostringstream ret;
	for(string::iterator i =in.begin(); i!=in.end(); ++i){
		if(*i!='%')
			ret<<*i;
		else{
			++i;
			string str(i,i+2);
			ret<<(unsigned char)strtol(str.c_str(),0,16);
			++i;
		}
	}
	return ret.str();
}

http_hdr::http_hdr(http_command command, string filename):fname(filename)
{
	ostringstream hdr;
	hdr<<str_http_cmd[command]<<" "<<conn_ctx.path<<to_persent_byte(filename)<<" HTTP/1.1"<<endl;
//	hdr<<str_http_cmd[command]<<" "<<conn_ctx.path<<filename<<" HTTP/1.1"<<endl;
	header=hdr.str();
	values["Host"]=conn_ctx.host;
	if(!conn_ctx.creds.empty()){
		values["Authorization"]="Basic "+base64_encode(conn_ctx.creds);
	}

	sock=0;
	ssl_ctx=NULL;
	ssl=NULL;
	mode=CT_NORMAL;
	bio_err=NULL;

	data=NULL;
	data_size=0;
	rdata=NULL;
	conn_state=WD_DISCONNECTED;
}

void http_hdr::add_body(const void* buf, size_t len)
{
	data=new char[len];
	memcpy(data, buf, len);
	data_size=len;
	ostringstream _cl;
	_cl<<len;
	values["Content-Length"]=_cl.str();
}

void http_hdr::add_lt(std::string lock_token)
{
	if(!lock_token.empty()){
		values["If"]="(";
		values["If"]+=lock_token+")";
	}
}

static void sigpipe_handle(int s)
{
	return;
}

int http_hdr::check_cert(SSL* conn,const  char* host)
{
	X509 *cert;
//	X509_NAME *xname;
//	int ret=-1;
//	char cname[256];
	long vresult;
	if (conn == NULL)
		return -1;	/* Нифига не SSL */
	cert = SSL_get_peer_certificate(conn);
	if (!cert){/* Нет сертификата */
		ostringstream msg;
		msg<<"Ошибка проверки сертификата сервера : сертификат отсутствует";
		//conn_ctx.ipc->send(ipc_proxy::NOTE_MSG, conn_ctx.pathNotify.c_str(), msg.str().c_str());
		conn_ctx.ipc->send(PSYNCNOTIFY_NOTIFICATION_MESSAGE, PSYNCNOTIFY_IPC_NAME, msg.str());
		return -1;
	}
	vresult = SSL_get_verify_result(conn);
	if ((vresult != X509_V_OK) &&
		(vresult != X509_V_ERR_DEPTH_ZERO_SELF_SIGNED_CERT)){/* Громко ругаемся. */
		ostringstream msg;
		msg<<"Ошибка проверки сертификата сервера : неверный сертификат : "<< vresult;
		//conn_ctx.ipc->send(ipc_proxy::NOTE_MSG, conn_ctx.pathNotify.c_str(), msg.str().c_str());
//TODO: 	conn_ctx.ipc->send(PSYNCNOTIFY_NOTIFICATION_MESSAGE, PSYNCNOTIFY_IPC_NAME, msg.str().c_str());
		return -2;
	}
//	xname = X509_get_subject_name(cert);
//	cname[0] = '\0';
//	X509_NAME_get_text_by_NID(xname, NID_commonName,cname, sizeof(cname));
//
//	if (!cname_match(cname, host))
//		/* Нас пытались обмануть */

    /* TODO:Еще надо проверить ASN1 */
    /* Позвать *_free() */
	X509_free(cert);
    /* Все хорошо. */
	return (0);
}
void http_hdr::connect()
{
	if(conn_state==WD_CONNECTED)
		return;
	sockaddr_in name;
	name.sin_family = AF_INET;
	name.sin_port   = htons( conn_ctx.port );
	hostent* hoste = gethostbyname(conn_ctx.host.c_str());
	if (hoste == NULL) {
		if( (hoste =gethostbyname("sync1.mandrivasync.com")) == NULL )
			throw "cannot find host to connect";
	}
	memcpy(&name.sin_addr,hoste->h_addr, hoste->h_length);
	if(conn_state==WD_EPIPE){
		//TODO: sess=SSL_get_session(ssl)
		SSL_shutdown(ssl);
		SSL_free(ssl);
		close(sock);

		conn_state=WD_DISCONNECTED;
	}

	sock=socket(AF_INET, SOCK_STREAM, 0);
	int err = ::connect(sock, (sockaddr*)&name, sizeof(sockaddr_in));
	if(err){
		perror("connect error");
		throw "connect error";
	}
	if(conn_ctx.ssl){
		if(!ssl){
			bio_err=0;
			SSL_METHOD *meth;

			if(!bio_err){
				SSL_library_init();
				SSL_load_error_strings();

				bio_err=BIO_new_fp(stderr,BIO_NOCLOSE);
			}

			/* Set up a SIGPIPE handler */
			signal(SIGPIPE,sigpipe_handle);

			meth=(SSL_METHOD*)SSLv3_method();
			ssl_ctx=SSL_CTX_new(meth);
		}

		ssl=SSL_new(ssl_ctx);
		sbio=BIO_new_socket(sock,BIO_NOCLOSE);
		SSL_set_bio(ssl,sbio,sbio);

		if(SSL_connect(ssl)<=0)
			throw "SSL connect error";
		check_cert(ssl,conn_ctx.host.c_str());
	}

	conn_state=WD_CONNECTED;
}
int http_hdr::_send(const void* buf, size_t len)
{
	int ret=HH_FAIL;
	int r;
	if(!len)
		return HH_OK;
	
	if(conn_ctx.stopflag)
		throw "Sync emergency stop";

	if(conn_ctx.ssl){
		if(conn_state!=WD_CONNECTED)
			connect();
		ERR_clear_error();
		r=SSL_write(ssl, buf, len);

		switch(SSL_get_error(ssl,r)){
			/* We wrote something*/
			case SSL_ERROR_NONE:
				break;
			case SSL_ERROR_ZERO_RETURN:
				conn_state=WD_EPIPE;
				ret=HH_RETRY;
				break;
			case SSL_ERROR_SYSCALL:
				if(r==-1 && errno == EPIPE){
					conn_state=WD_EPIPE;
					ret=HH_RETRY;
					break;
				}
		  	default:
				cerr<<"Unrecoveroble SSL write error :"<< SSL_get_error(ssl,r)<<":"<<r<<":"<<errno<<endl;
				return HH_FAIL;
		}
	} else {
		r=::send(sock, buf, len, MSG_NOSIGNAL);
		if(r==-1 &&
			(errno==ENOTCONN || errno==EPIPE)){
			conn_state=WD_EPIPE;
			ret=HH_RETRY;
		}
	}
	if(r > 0 && (unsigned)r==len)
		ret=HH_OK;
#ifdef DEBUG
	if(r>0){
		ofstream slog((conn_ctx.home+"serverlog").c_str(),ios_base::app|ios_base::out);
		if(slog.is_open())
			slog<<endl<<"\033[01;34m"<<string((char*)buf,r)<<"\033[00m"<<endl;
	}
#endif
	return ret;
}
int http_hdr::_recv(void* buf, size_t len)
{
	int r=HH_FAIL;

	if(conn_ctx.stopflag)
		throw "Sync emergency stop";

	while(len){
		if(conn_ctx.ssl){
			ERR_clear_error();
//			if(pending()<0)
//				return HH_FAIL;
			r=SSL_read(ssl,buf,len);
			switch(SSL_get_error(ssl,r)){
				case SSL_ERROR_NONE:
			      		break;
				case SSL_ERROR_ZERO_RETURN:
					conn_state=WD_EPIPE;
					return HH_RETRY;
				case SSL_ERROR_WANT_READ:
			      		throw "read_blocked=1";
			      		break;
				case SSL_ERROR_WANT_WRITE:
					throw "read_blocked_on_write=1";
			      		break;
				case SSL_ERROR_SYSCALL:
					if(r==-1 && errno == EPIPE){
						conn_state=WD_EPIPE;
						return HH_RETRY;
					}

			    	default:
					cerr<<"Unrecoveroble SSL read error :"<< SSL_get_error(ssl,r)<<":"<<r<<":"<<errno<<endl;
					conn_state=WD_EPIPE;
					return HH_RETRY;
			}
		} else {
			r=::recv(sock, buf, len, 0);
			if(r==0){
				conn_state=WD_EPIPE;
				return HH_RETRY;
			}
		}
#ifdef DEBUG
		{
			ofstream slog((conn_ctx.home+"serverlog").c_str(),ios_base::app|ios_base::out);
			if(slog.is_open())
				slog<<endl<<"r: "<<r<<" \033[01;31m" <<string((char*)buf,r)<<"\033[00m"<<endl;
		}
#endif
		len-=r;
		buf=(char*)buf+r;
	}
	return HH_OK;
}

http_hdr::proto_ret http_hdr::send()
{
	http_hdr::connect();
	int ret;

	//Собираем полностью всё
	string head=toString();
	if(mode != CT_FILEIN){ //НЕ ФАЙЛ
		char buf[data_size+head.length()];
		memcpy(buf, head.c_str(), head.length());
		if(data_size)
			memcpy(buf+head.length(), data, data_size);
		//Посылаем
		ret=_send(buf, data_size+head.length());
		if(ret) return (proto_ret)ret;
	}else {				//CT_FILEIN
		ret=_send(head.c_str(),head.length());
		if(ret) return (proto_ret)ret;
		char buf[SSL_BUFSIZE];
		long long transmitted=0;
		int mcl=atoi(values["Content-Length"].c_str());

		do
		{
			ifd->read(buf,SSL_BUFSIZE);
			ret =_send(buf,ifd->gcount());
			if(ret) {
				ifd->seekg(0);
				conn_ctx.ipc->send(PSYNCNOTIFY_FILESEND_COMPLETED, PSYNCNOTIFY_IPC_NAME, string("/")+fname);
				return (proto_ret)ret;
			}
			//IPC
			if(mcl){
				transmitted+=ifd->gcount();
				ostringstream msg;
				int pt= (transmitted*100)/mcl;
				msg.write((char*)&pt,sizeof(int));
				msg<<"/"<<fname;
				conn_ctx.ipc->send(PSYNCNOTIFY_FILESEND_PROGRESS, PSYNCNOTIFY_IPC_NAME, msg.str());
			}
			//\IPC
		}
		while( !ifd->eof());
		conn_ctx.ipc->send(PSYNCNOTIFY_FILESEND_COMPLETED, PSYNCNOTIFY_IPC_NAME, string("/")+fname);
	}

	try{
		ret=recv();
	}catch(http_exception::moved){					// Меняем сервер
			conn_ctx.host=ivalues["Location"];
			string::size_type i=conn_ctx.host.find("//")+2;
			conn_ctx.host=conn_ctx.host.substr(i, conn_ctx.host.find("/",i)-i);
			throw;
	}catch(http_exception::auth_err){		//Если 401 - мочим логин и пароль
			unlink((conn_ctx.home+"cfg/user/login").c_str());
			unlink((conn_ctx.home+"cfg/user/password").c_str());
			throw;
	}
	return (proto_ret)ret;
}

int  http_hdr::pending()
{
	fd_set fds;
	struct timeval tv;
	int r;
	FD_ZERO (&fds);
	FD_SET (sock, &fds);
	/* Timeout. */
	tv.tv_sec = 5;
	tv.tv_usec = 0;

	r = select (sock+1, &fds, NULL, NULL, &tv);

	if (-1 == r) {
		if (EINTR == errno)
			throw "Socket select error";
	}
//	if (0 == r) 		//Убрано Для контроля за состоянием соединения
//		return 0;

	if(ssl){
		int r=SSL_peek(ssl,&r,1);
		switch(SSL_get_error(ssl,r)){
			case SSL_ERROR_NONE:
				break;
			case SSL_ERROR_ZERO_RETURN:
				conn_state=WD_EPIPE;
				return -HH_RETRY;
			case SSL_ERROR_WANT_READ:
				throw "read_blocked=1";
				break;
			case SSL_ERROR_WANT_WRITE:
				throw "read_blocked_on_write=1";
				break;
			case SSL_ERROR_SYSCALL:
				if(r==-1 && errno == EPIPE){
					conn_state=WD_EPIPE;
					return -HH_RETRY;
					break;
				}
			default:
				cerr<<"Unrecoveroble SSL read error :"<< SSL_get_error(ssl,r)<<":"<<r<<":"<<errno<<endl;
				return -HH_FAIL;
		}
		return SSL_pending(ssl);
	}else{
		int pending;
		if(ioctl(sock,FIONREAD,&pending)==0)		//TODO: проверка на обрыв и ретрай
			return pending;
		else return -HH_RETRY;
	}

}

void http_hdr::parse_rcvd(string _s)
{
	status=-1;
	if(rdata){
		delete[] rdata;
		rdata=NULL;
	}
	ivalues.clear();
	end=_s.find("\r\n\r\n");
	istringstream s(_s.substr(0, end));
	if(end!=string::npos)
		end+=4;
	string line;
	getline(s, line);
	string::size_type i;
	i=line.find("HTTP/1.1 ");
	if(i==string::npos)
		return;
	status=atoi(line.substr(i+9,string::npos).c_str());
	while(getline(s,line)){
		i=line.find(":");
		if(i==string::npos)
			return;
		ivalues[line.substr(0,i)]=line.substr(i+2);
	}
	more_data_size=cl=atoi(ivalues["Content-Length"].c_str());
	if(mode==CT_FILEOUT){
		if(more_data_size>HTTP_HDR_MAX)
			more_data_size=HTTP_HDR_MAX;
	}
	if(cl==0 && (ivalues["Transfer-Encoding"].find("chunked")!=string::npos)){ 		//CHUNKED
#warning TODO:chunked header processing
		//istringstream s(_s.substr(end));
		//unsigned ch_size;
		//s>>ch_size;
		//
		//
	}else{											//CONTENT-LENGTH not NULL
		more=more_data_size;
		more_data=rdata=new char[more_data_size];
		if((_s.length()-end)> more_data_size)
			throw "Content size mismatch";
		memcpy(more_data,_s.c_str()+end,_s.length()-end);
		more_data+=_s.length()-end;
		more-=_s.length()-end;
		if(more)
			_recv(more_data, more);		//получили данные до метра
	}

	http_exception::error::check(status);
}

http_hdr::proto_ret http_hdr::recv()
{
	int ret=HH_RETRY;
	int sz=pending();
	if(sz<0)
		return (proto_ret)-sz;
	char rcvbuf[sz];
	ret = _recv( rcvbuf, sz);
	if(ret) return (proto_ret)ret;
	parse_rcvd(string(rcvbuf,sz));
	//TODO? if(ret) return ret;
	if(mode == CT_FILEOUT){
		unsigned size=cl;
		ofd->write(rdata, more_data_size);
		size-=more_data_size;

		if(cl){
			ostringstream msg;
			long long transferred=cl-size;
			transferred*=100;
			int pt=(transferred/cl);
			msg.write((char*)&pt,sizeof(int));
			msg<<"/"<<fname;
			conn_ctx.ipc->send(PSYNCNOTIFY_FILERECV_PROGRESS, PSYNCNOTIFY_IPC_NAME, msg.str());
		}
		//заливаем файл
		while(size)
		{
			char buf[SSL_BUFSIZE];
			unsigned sz=size>SSL_BUFSIZE?SSL_BUFSIZE:size;
			if(_recv(buf,sz)!=HH_OK){
				cerr<<"File save error"<<endl;
				cerr<<"File corrupted"<<endl;
				return HH_FAIL;
			}
			ofd->write(buf,sz);
			if(!ofd){
				cerr<<"File save error"<<endl;
				cerr<<"File corrupted"<<endl;
				return HH_FAIL;
			}
			size-=sz;

			ostringstream msg;
			long long transferred=cl-size;
			transferred*=100;
			int pt=(transferred/cl);
			msg.write((char*)&pt,sizeof(int));
			msg<<"/"<<fname;
			conn_ctx.ipc->send(PSYNCNOTIFY_FILERECV_PROGRESS, PSYNCNOTIFY_IPC_NAME, msg.str());
		}
		conn_ctx.ipc->send(PSYNCNOTIFY_FILERECV_COMPLETED, PSYNCNOTIFY_IPC_NAME, string("/")+fname);
	}
	return HH_OK;
}

string http_hdr::toString()
{
	string ret;
	ret=header;
	for(map<string,string>::const_iterator p=values.begin();p!=values.end();p++)
		ret+=p->first+": "+p->second+"\n";
	ret+="\n";
	return ret;
}

int http_hdr::process_request()
{
	proto_ret err;
	int retries=2; //Один если упало соединение и один для 301 Moved
	if(mode==CT_NORMAL)
		values["Content-Type"] = "text/xml; charset=\"utf-8\"";
	do{
		err = send();
		retries--;
	}while(err==HH_RETRY && retries);
	if(mode!=CT_NORMAL){
		if(err!=HH_OK){
			throw "File operation retry excedeed";
		}else{
		//TODO:	ostringstream msg;
		//	msg<<"Обработка файла "<<fname<<" завершена успешно" ;
		//	conn_ctx.ipc->send(PSYNCNOTIFY_NOTIFICATION_MESSAGE, PSYNCNOTIFY_IPC_NAME, msg.str().c_str());
		}

	}

	return err;
}
void http_hdr::add_file_body(std::ifstream* fd, size_t size)
{
	ifd=fd;
 	data_size=size;
	mode=CT_FILEIN;
	ostringstream _cl;
	_cl<<size;
	values["Content-Length"]=_cl.str();
}

void http_exception::error::check(int err)
{
	if((err>=200) && (err<300))
		return;//OK
	switch(err)
	{
		case 301:
			throw http_exception::moved();
		case 401:
			throw http_exception::auth_err();
		case 403:
			throw http_exception::forbidden();
		case 404:
			throw http_exception::not_found();
		case 405:
			throw http_exception::not_allowed();
		case 409:
			throw http_exception::conflict();
		default:
			throw http_exception::error(err);
	}
}
