#ifndef FILE_SYNC_H 
#define FILE_SYNC_H
#include <map>
#include <string>
#include <sstream>
#include <sys/types.h>

using namespace std;
class filedb;
class WebDav;

class sync_object
{
private:
	enum files_flags
	{
		FL_DELETE=1,
		FL_DELETED=2,
		FL_LEVEL2=4,
	};
	enum udp_state
	{
		SO_UPTODATE,
		SO_CREATE,
		SO_PUSH,
		SO_PULL,
		SO_DELETE,
		SO_CONFLICT,
	};

	std::string uname;	//!уникальное имя для хранения на сервере 
	std::string path;	//!путь на локальной системе
	std::string sTag;       //!sТаг на сервере
	time_t sync_time;	//!время синхронизации
	mode_t mode;		//!Права файла
	char sync_point;	//!Принадлежность к точке синхронизации
	char flags;		//!Флаги file_flags
	
	size_t server_size;

	int inotify_fd;
	int state;
	filedb* db;

	int get(WebDav* dav);
	int put(WebDav* dav);
	void genSTag();
	time_t GetMTime();

	int put_subdir(string path, string uname, WebDav* dav);
	int get_subdir(string path, string uname, WebDav* dav);
	int checkSubdir(WebDav* dav);
	sync_object* getUpDir();

	void checkState(WebDav* dav);

	map<string,string> getMetaMap() const;
public:
	sync_object(string path, string uname);
	sync_object(string _uname, string _path,time_t _sync_time, string _stag,
			short _mode, char _sync_point, char _flags, size_t serversize=0);

	sync_object(string _l);
	sync_object();
	~sync_object();

	void setDB(filedb* _db){db=_db;};
	void leveldown(){flags|=FL_LEVEL2;};
	int level() const {return flags&FL_LEVEL2;};
	void set_ifd(int fd){inotify_fd=fd;};
	int get_ifd(){return inotify_fd;};
	int valid();

	int operator==(string s);
	int operator==(sync_object s);
	bool inDir(string path);

	int update(WebDav* dav);
static  void mkdirto(string path);

	void getData(string& uname, string& path,time_t& sync_time, string& stag, mode_t& mode, char& sync_point, char& flags) const; 
	string getStringData() const;
	string getPath() const {return path;};
	string getUName() const {return uname;};

	void setDelete(){flags|=FL_DELETE;};
	void undelete(){flags^=FL_DELETE;};
	bool toDelete(){return flags&FL_DELETE;};
	bool canDelete(){return flags&FL_DELETED;};
};

#endif
