#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <errno.h>
#include <poll.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <ipcpacket.h>
#include "ipc_proxy.h"


ipc_proxy::ipc_proxy(const char* path):spath(path)
{
	sock=socket(AF_LOCAL, SOCK_DGRAM, 0);
	if(sock==-1){
		perror("ipc_socket");
		throw "Ошибка создания UNIX-сокета";
	}
	struct sockaddr_un addr;
	memset(&addr, 0, sizeof(sockaddr_un));
	addr.sun_family=AF_UNIX;
	strncpy(addr.sun_path, path, sizeof(addr.sun_path)-1);
	unlink(path);
	if(bind(sock,(sockaddr*)&addr,sizeof(sockaddr_un))==-1){
		perror("ipc_socket bind");
		throw "Ошибка создания сокета взаимодействия  приложений синхронизации\nВозможно отсутствует папка ~/.psyncclient";
	}
}

ipc_proxy::~ipc_proxy()
{
	close(sock);
	unlink(spath.c_str());
}

int ipc_proxy::next()
{
    IPCPacket *m_pkt = NULL;
    size_t bytes=0;
    int ret=0;
    ioctl(sock, FIONREAD, (char *) &bytes);

    if (!bytes)
        return -EAGAIN;

    m_pkt = new IPCPacket();
    recv(sock,(char*)m_pkt->readBuffer(bytes), bytes,0);

    if (m_pkt->IsValid())
    {               
    	fpath=m_pkt->getSender();
        switch (m_pkt->getType())
        {
		case SYNCRONIZATION_STATUS:
//		case PSYNCD_SYNCRONIZATION_STATUS:
			if(memcmp((const char*)m_pkt->getData(),STOP_SYNCRONIZATION,4))
				state=SYNC_START;
			else
				state=SYNC_STOP;
			break;
		case PSYNCD_SYNC_START:
			state=SYNC_START;
			break;
		case PSYNCD_SYNC_STOP:
			state=SYNC_STOP;
			break;
		case PSYNCD_GET_SYNC_STATUS:
//		case PSYNCD_GET_SYNC_STATUS:
			state=SYNC_STATUS;
			break;
		case ADD_TO_SYNC_COMMAND:
//		case PSYNCD_ADD_TO_SYNC_COMMAND:
			state=FILE_ADD;
			data=std::string((char*)m_pkt->getData(),m_pkt->getDataLen());
			break;
		case REMOVE_TO_SYNC_COMMAND:
			state=FILE_DEL;
			data=std::string((char*)m_pkt->getData(),m_pkt->getDataLen());
			break;

		case GET_FILES_FROM_FOLDER_COMMAND:
			state=DIR_STATUS;
			data=std::string((char*)m_pkt->getData(),m_pkt->getDataLen());
			break;
		case PUT_SYNC_FILES:
			state=DIR_STATUS;
			data=std::string((char*)m_pkt->getData(),m_pkt->getDataLen());
			break;
		case GET_ALL_FILES_COMMAND:
			state=LIST_ALL;
			data=std::string((char*)m_pkt->getData(),m_pkt->getDataLen());
			break;


		case REQUEST_LOGIN_AND_PASSWORD:
			state=REQ_LOGIN;
			data=std::string((char*)m_pkt->getData(),m_pkt->getDataLen());
			break;

		case ACCOUNT_VALID:
			state=ACC_VALID;
			data=std::string((char*)m_pkt->getData(),m_pkt->getDataLen());
			break;

		case PASSWORD_CHANGED:
			state=PWD_CHANGED;
			data=std::string((char*)m_pkt->getData(),m_pkt->getDataLen());
			break;
		case SETTINGS_CHANGED:
			state=SET_CHANGED;
			data=std::string((char*)m_pkt->getData(),m_pkt->getDataLen());
			break;

		default:
			ret = -EIO;
        }
    }

    if (m_pkt)
        delete m_pkt;
    return ret;

}

int ipc_proxy::nextb()
{
	int err;
	while((err=next())==-EAGAIN)
	{
		poll(0,0,100);
	}
	return err;
}

std::string ipc_proxy::getData()
{
	return data;
}

int ipc_proxy::send_old(status st, const char* path, std::string pdata)
{
	/*struct sockaddr_un clnt_addr;
	clnt_addr.sun_family = AF_UNIX;
	strcpy(clnt_addr.sun_path, path);
	int clnt_addr_len = sizeof(clnt_addr);



	const char* data;
	unsigned datalen;
	int type;
	switch(st)
	{
		case SYNC_START:
			data = START_SYNCRONIZATION;
			datalen = 5;
			type = SYNCRONIZATION_STATUS;
			break;
		case SYNC_STOP:
			data = STOP_SYNCRONIZATION;
			datalen = 4;
			type = SYNCRONIZATION_STATUS;
			break;
		case NOTE_MSG:
			type=NOTIFICATION_MESSAGE;
			data = pdata.c_str();
			datalen = pdata.length();
			break;
		case SYNC_STATUS:
			type=PSYNCD_GET_SYNC_STATUS;
			data = pdata.c_str();
			datalen = pdata.length();
			break;
		case DIR_STATUS:
			type = PUT_SYNC_FILES;
			data = pdata.c_str();
			datalen = pdata.length();
			break;
		case FILE_ADD:
			type = ADD_TO_SYNC_COMMAND;
			data = pdata.c_str();
			datalen = pdata.length();
			break;
		case FILE_DEL:
			type = REMOVE_TO_SYNC_COMMAND;
			data = pdata.c_str();
			datalen = pdata.length();
			break;
		case ACC_VALID:
			type=ACCOUNT_VALID;
			data = pdata.c_str();
			datalen = pdata.length();
			break;
		case PWD_CHANGED:
			type=PASSWORD_CHANGED;
			data = pdata.c_str();
			datalen = pdata.length();
			break;

		case LIST_ALL:
			type=GET_ALL_FILES_COMMAND;
			data = pdata.c_str();
			datalen = pdata.length();
			break;
		default:
			return -EINVAL;
	}

    IPCPacket* pkt = IPCPacket::Create(type, (char*)spath.c_str(), (unsigned char*)data, (unsigned long)datalen);
    if ( sendto(sock, pkt->getBuffer(), pkt->getSize(), 0, (struct sockaddr *)&clnt_addr, (socklen_t)clnt_addr_len) != (unsigned)pkt->getSize()){
	    std::cerr<<"ipc_proxy: send error; Dest:"<<path <<std::endl;
	    delete pkt;
	    return -EIO;
    }
    delete pkt;*/
    return 0;
}

int ipc_proxy::send(int type, std::string receiver, std::string pdata)
{
    struct sockaddr_un clnt_addr;
    clnt_addr.sun_family = AF_UNIX;

    if(receiver[0]!='/')	receiver = getenv("HOME")+std::string("/")+receiver;
    strcpy(clnt_addr.sun_path, receiver.c_str());
    int clnt_addr_len = sizeof(clnt_addr);

    const char* data = pdata.c_str();
    unsigned datalen = pdata.length();

    //std::cout<<"Sending msg: "<<type<<" to: "<<receiver<<" datalen: "<<datalen<<std::endl;
    IPCPacket* pkt = IPCPacket::Create(type, (char*)spath.c_str(), (unsigned char*)data, (unsigned long)datalen);
    if ( sendto(sock, pkt->getBuffer(), pkt->getSize(), 0, (struct sockaddr *)&clnt_addr, (socklen_t)clnt_addr_len) != (unsigned)pkt->getSize()){
	    std::cerr<<"ipc_proxy: send error; Dest:"<<receiver<<std::endl;
	    delete pkt;
	    return -EIO;
    }
    delete pkt;
    return 0;
}
