#include <sstream>
#include <pthread.h>
#include <sys/poll.h>
#include <sys/ioctl.h>
#include <sys/inotify.h>
#include <errno.h>
#include <iostream>
#include "sync.h"
#include <libcfg.h>
#include "http_hdr.h"

extern http_ctx conn_ctx;
syncronizer::syncronizer(string srv,unsigned short port, string col)
{
	is_working=0;
	stopflag=0;
	automode=0;
	pthread_spin_init(&lock,0);
	sync_time=0;
	def_synctime=0;
	mode=MODE_NONE;
}

syncronizer::~syncronizer()
{
	stop();
	stopAuto();
	pthread_spin_destroy(&lock);
}

static void* thread(void* arg)
{
	syncronizer* ctx=(syncronizer*) arg;
	ctx->fthread();
	return NULL;
}

static void* athread(void* arg)
{
	syncronizer* ctx=(syncronizer*) arg;
	ctx->athread();
	return NULL;
}

void syncronizer::fthread()
{
	while(is_working){
		if(mode!=MODE_TIME)
			poll(0,0,100);
		if(sync_time>0){
			if(!(--sync_time))
				--sync_time;
			poll(0,0,1000);
		}else if(sync_time == 0)
			poll(0,0,100);
		else {
			sync_time=def_synctime;
			sync();
		}
	}
}

static vector<sync_object> parseServerTree(WebDav* dav)
{
	vector <sync_object> srv_files = dav->GetServerList();
	vector <sync_object> ret;
	while(!srv_files.empty())
	{
		sync_object so=srv_files.back();
		srv_files.pop_back();
		string uname=so.getUName();
		if(uname[uname.length()-1]=='/'){
			vector <sync_object> srv_files1 = dav->GetServerList(uname);
			srv_files.insert(srv_files.end(),srv_files1.begin(),srv_files1.end());
		}
		if(so.valid())
			ret.push_back(so);
	}
	return ret;
}

int syncronizer::sync()
{

	WebDav dav;
	conn_ctx.stopflag=0;
	ipc->send(PSYNCNOTIFY_SYNC_START, PSYNCNOTIFY_IPC_NAME);
	ipc->send(PSYNCCONFIG_SYNC_START, PSYNCCONFIG_IPC_NAME);
	try{
		pthread_spin_lock(&lock);
		db.lock();
		vector <sync_object> srv_files = parseServerTree(&dav);
		db.combine(srv_files);
		db.unlock();
		sync_object::mkdirto(conn_ctx.defFolder);

		sync_object* i=NULL;
		while(db.next(i)){
			if(conn_ctx.stopflag)
				break;
			try{
				i->update(&dav);
			}catch(http_exception::error err){
				std::cerr<<"File sync error: "<<err.status()<<std::endl;
				ostringstream msg;
				msg<<err.status();
				ipc->send(PSYNCNOTIFY_HTTP_ERROR, PSYNCNOTIFY_IPC_NAME, msg.str());
			}
		}
	
		db.lock();
		db.delDeleted();
		db.save();
		pthread_spin_unlock(&lock);
		db.unlock();
		std::cerr<<"Sync success"<<std::endl;
		set_param_ui((conn_ctx.home+"cfg/user/").c_str(),"lastsync",time(NULL));
		long long max, cur;
		if(!dav.GetQuote(max, cur)){
			set_param_ull((conn_ctx.home+"info/").c_str(),"quota_total",max);
			set_param_ull((conn_ctx.home+"info/").c_str(),"quota_used",cur);
		}
	}catch(string str){
		std::cerr<<"Sync thread critical error: "<<str<<std::endl;
		db.unlock();
	}catch(const char* str){
		std::cerr<<"Sync thread critical error: "<<str<<std::endl;
		db.unlock();
		pthread_spin_unlock(&lock);
	}catch(http_exception::auth_err){		//Если 401 - мочим логин и пароль
		unlink((conn_ctx.home+"cfg/user/login").c_str());
		unlink((conn_ctx.home+"cfg/user/password").c_str());
		db.unlock();
		pthread_spin_unlock(&lock);				
	}catch(http_exception::moved){					// Меняем сервер
		db.unlock();
		pthread_spin_unlock(&lock);				
    		set_param_str((conn_ctx.home+"server/").c_str(), "address", (char*)conn_ctx.host.c_str()); // Сохраняем
		syncAll();						// И перезапускаем
	}catch(http_exception::error err){
		std::cerr<<"Sync thread HTTP critical error: "<<err.status()<<std::endl;
		db.unlock();
		pthread_spin_unlock(&lock);
	}
	ipc->send(PSYNCNOTIFY_SYNC_STOP, PSYNCNOTIFY_IPC_NAME);
	ipc->send(PSYNCCONFIG_SYNC_STOP, PSYNCCONFIG_IPC_NAME);

	return 0;
}

int syncronizer::TryAuth()
{
	try{
		return WebDav().TryAuth();
	}catch(http_exception::moved){
		return WebDav().TryAuth();
	}
}

int syncronizer::dbconnect(string path)
{
	db.dbset(path);
	db.load();
	return 0;
}

int syncronizer::addFile(string path)
{
	stopAuto();
 	db.insert(path);
	startAuto();
	return 0;
}

int syncronizer::removeFile(string path)
{
	stopAuto();
 	db.remove(path);
	startAuto();
	return 0;
}

int syncronizer::checkFile(string path)
{
	if(db.find(path))
		return 1;
	else 
		return 0;
}

string syncronizer::checkDir(string path)
{
	stringstream ret;
	sync_object* i=NULL;
	while(db.next(i))
		if(i->inDir(path) && !i->toDelete()){ 
			if(i->level())
				ret<<"+";
			ret<<i->getPath()<<'\n';
		}
	return ret.str();
}

string syncronizer::listAll()
{
	stringstream ret;
	sync_object* i=NULL;
	while(db.next(i)){
		if(!(i->toDelete()))
			ret<<i->getPath()<<'\n';
	}
	return ret.str();
}


int syncronizer::is_work()
{
	int ret=pthread_spin_trylock(&lock);
	if(!ret)
	{
		pthread_spin_unlock(&lock);
	}
	return !!ret;
}

void syncronizer::setSyncTime(int sec)
{
	unsigned lasttm=0;
	get_param_ui((conn_ctx.home+"cfg/user/").c_str(),"lastsync",&lasttm);
	unsigned currtm=time(NULL);

	cerr<<"config sync period="<<sec<<endl;
	cerr<<"last synctime="<<lasttm<<endl;
	cerr<<"current synctime="<<currtm<<endl;

	sync_time=sec;
	def_synctime=sec;
	if(sec==0) return; // zero parameter from config - sync is off

	if(lasttm==0)
	{
		sync_time = sec;
		cerr<<"timer!: very first sync, set to "<<sync_time<<endl;
		return;
	}

	if(lasttm>=currtm) // something strange -do it now = update last sync time (TODO: think, think, think!)
	{
		cerr<<"timer!: hello future !! well proceed in 1 sec"<<endl;
		sync_time=1;
		return;
	}

	if((currtm-lasttm) >= (unsigned)sync_time) // period already elapsed since last sync time - start sync after 1 sec pause
	{
		cerr<<"timer!: already must do it, wait 1 sec and proceed!"<<endl;
		sync_time=1;
		return;
	}

	sync_time-=(currtm-lasttm);  // just take into account elapsed time
	cerr<<"timer!: set to "<<sync_time<<endl;
}

void syncronizer::syncAll()
{
	sync_time=-1;
}
void syncronizer::setMode(int _mode)
{
	mode=_mode;
}
int syncronizer::start()
{
	if(is_working)
		return -1;
	is_working=1;
	pthread_create(&th, NULL, thread, this);
	return 0;
}
void syncronizer::syncStop()
{
	conn_ctx.stopflag=1;
}
void syncronizer::stop()
{
	is_working=0;
	pthread_join(th,NULL);
}

void syncronizer::startAuto()
{
	if(mode!=MODE_AUTO)
		return;
	sync_time=0;
	automode=1;
	pthread_create(&ath, NULL, ::athread, this);
}

void syncronizer::athread()
{
	int ifd;
	int pending;
	db.lock();
	WebDav dav;
	try{
		ifd= inotify_init1(IN_NONBLOCK);
		if(ifd==-1)
			throw string("Ошибка получения дескриптора inotify");
		sync_object* i=NULL;
		while(db.next(i)){
			int watch = inotify_add_watch(ifd, i->getPath().c_str(), IN_CREATE|IN_DELETE|IN_DELETE_SELF|IN_MODIFY );
			if(watch == -1)
				throw string("Ошибка добавления файла к inotify");
			else
				i->set_ifd(watch);
		}
		while(automode)
		{
			if(ioctl(ifd, FIONREAD,&pending)!=0)
				throw string("Ошибка ioctl FIONREAD");
			if(!pending){
				poll(0,0,100);
				continue;
			}
			char wbuf[pending];
			int cnt;
			cnt=read(ifd, wbuf, pending);
			if(cnt!=pending)
				throw string ("Ошибка чтения из inotify дескриптора");
			cnt=0;
			while(pending>0){
				inotify_event* ie=(inotify_event*)(((char*)wbuf)+cnt);
				cnt+=sizeof(inotify_event)+ie->len;
				pending-=sizeof(inotify_event)+ie->len;
				sync_object* obj=db.find_wd(ie->wd);

				obj->update(&dav);

				if(obj->canDelete())
					db.erase(obj);
			}
		}
		close(ifd);
	}catch(string str)
	{
		close(ifd);
		db.unlock();
		automode=0;
		std::cerr<<"Autosync thread critical error: "<<str<<std::endl;
		return ;
	}
	db.unlock();

	return ;
}

void syncronizer::stopAuto()
{
	if(!automode)
		return;
	automode=0;
	pthread_join(ath,NULL);
}

