#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <sstream>
#include <map>
#include <iostream>
#include <iomanip>
#include <errno.h>
#include <cstdlib>
#include <openssl/md5.h>
#include <sys/statfs.h>
#include <time.h>
#include "sync_obj.h"
#include "WebDAV.h"
#include "filesdb.h"

sync_object::sync_object(string _path,string _uname):path(_path),sync_time(0)
{
//	uname=path.substr(path.rfind("/")+1);
//	uname=_uname+uname;
	uname=path.substr(1);	//Убираем первый '/'
	sync_point=-1;
	struct stat st;
	if(stat(path.c_str(), &st)!=-1){
		mode=st.st_mode;
		if(S_ISDIR(mode))
			uname+="/";
	}else{
		mode=0;
	};//TODO: Сделать чтонибудь если файла нет
	flags=0;
	db=NULL;
	state=SO_UPTODATE;
}

sync_object::sync_object(string _uname, string _path,time_t _sync_time, string _stag,
		short _mode, char _sync_point, char _flags, size_t  serversize):uname(_uname),
	path(_path),sTag(_stag),sync_time(_sync_time),
		mode(_mode),sync_point(_sync_point),flags(_flags), server_size(serversize)
{ 
	db=NULL;
	state=SO_UPTODATE;
}

static string read_string(stringstream& _s)
{
	unsigned sz;
	_s.read((char*)&sz,sizeof(sz));
	char buf[sz];
	_s.read(buf,sz);
	return string(buf,sz);
}

sync_object::sync_object(string _l)
{
	stringstream s(_l);

	path=read_string(s);
	uname=read_string(s);
	sTag=read_string(s);
	s.read((char*)&sync_time,sizeof(sync_time));	
	s.read((char*)&mode,sizeof(mode));	
	s.read((char*)&flags,sizeof(flags));	
	s.read((char*)&sync_point,sizeof(sync_point)); 		
	db=NULL;
	state=SO_UPTODATE;

}
sync_object::sync_object()
{
	db=NULL;
	state=SO_UPTODATE;
}
sync_object::~sync_object()
{
	if(!db)
		return;
	if(!(flags&FL_DELETED))
		return;
	if(!S_ISDIR(mode))
		return;

	//Утаскиваем за собой потомков:)
	vector<sync_object*> sl=db->findDir(path);
	for(vector<sync_object*>::iterator i=sl.begin();i!=sl.end();++i){
		db->erase(*i);
	}

}

time_t sync_object::GetMTime()
{
	if(S_ISDIR(mode)){
		time_t stor=0;
		DIR* dir;
		dirent* idir;
		dir=opendir(path.c_str());
		if(dir == NULL)
			return 0;
		while((idir = readdir(dir)) != NULL ){
			if((string(idir->d_name)==".") || (string(idir->d_name) == "..") )
				continue;
			sync_object* nfile=db->find(path+"/"+string(idir->d_name));
			if(!nfile){
				closedir(dir);
				return time(NULL);
			}
			stor=stor>nfile->sync_time?stor:nfile->sync_time;
		}
		closedir(dir);
		return stor;
	}else{
		struct stat st;
		int ret;
		ret=stat(path.c_str(),&st);
		if(ret<0)
			return ret;
		return st.st_mtime>st.st_ctime?st.st_mtime:st.st_ctime;
	}
}


int sync_object::operator==(string s)
{
	return path==s;
}
int sync_object::operator==(sync_object s)
{
	return path==s.path;
}
bool sync_object::inDir(string s)
{
	if(s==path)
		return 0;
	return (path.find(s) == 0) && (path.find("/",s.length()+1) == string::npos);
}

void sync_object::genSTag()
{
	ostringstream s,s1;
	string tmp;
	s<<time(NULL);
	s<<rand();
	tmp = string(
		(char*)MD5( (const unsigned char*)s.str().c_str(), s.str().length() ,NULL),
		MD5_DIGEST_LENGTH);
	s1<<uppercase<<hex<<setw(2)<<setfill('0');

	for(unsigned  i=0; i<tmp.length(); i++)
		s1<<(unsigned int)((unsigned char)tmp[i]);
	sTag=s1.str();
}
int sync_object::checkSubdir(WebDav* dav)	//TODO: Принимать решение о удалении файлов локально/на сервере на основе state
{
	//Получаем список в папке
	DIR* dir;
	dirent* idir;
	dir=opendir(path.c_str());
	if(dir == NULL)
		return 0;
	while((idir = readdir(dir)) != NULL ){
		if((string(idir->d_name)==".") || (string(idir->d_name) == "..") )
			continue;
		db->insert(path+"/"+string(idir->d_name))->leveldown();
	}
	closedir(dir);
	//Получаем список на сервере
	vector<sync_object> list=dav->GetServerList(uname);
	db->combine(list);
	//Получаем список в базе
	vector<sync_object*> sl=db->findDir(path);
	for(vector<sync_object*>::iterator i=sl.begin();i!=sl.end();i++)
		(*i)->update(dav);
	return 0;
}

int sync_object::put(WebDav* dav)
{
	dav->Lock();
	try{
		if(S_ISDIR(mode)){
			dav->Mkcol(uname);
		}else{
			if(dav->Put(uname,path)){
				dav->Delete(uname);
				dav->Unlock();
				return -1;
			}
		}
		genSTag();
		dav->SetMeta(uname,getMetaMap());
	}catch(...){
	
		try{
			dav->Delete(uname);
		}catch(...){}
		conn_ctx.ipc->send(PSYNCNOTIFY_FILESEND_ERROR, PSYNCNOTIFY_IPC_NAME, path);
		dav->Unlock();
		throw;
	}
	sync_time=GetMTime();
	dav->Unlock();
	return 0;
}

void sync_object::mkdirto(string path)
{
	string p1=path.substr(0, path.rfind("/")+1);
	string::size_type i=0;
	while(true){
		i=p1.find("/",i+1);
		if(i==string::npos)
			break;
		string p2=p1.substr(0,i);
		int ret=mkdir(p2.c_str(),0755);
		if( ret == 0)
			continue;
		if(errno!= EEXIST)
			break;
	}
}

int sync_object::get(WebDav* dav)
{
	dav->Lock();
	mkdirto(path);
	//Size check
	struct statfs sfs;
	int ret=statfs(path.c_str(),&sfs);
	if(ret)
		ret=statfs(path.substr(0,path.rfind("/")).c_str(),&sfs);
	if(!ret){
		if(server_size > (sfs.f_bfree*sfs.f_bsize)){	//Size ERROR
			conn_ctx.ipc->send(PSYNCNOTIFY_FILERECV_NOSPACE, PSYNCNOTIFY_IPC_NAME, path);
			dav->Unlock();
			return -1;
		}
	}
	//End check
	try{
		if(S_ISDIR(mode)){
			mkdir(path.c_str(), mode);
		}else{
			dav->Get(uname,path);
			chmod(path.c_str(),mode);
		}
	}
	catch(...){
		dav->Unlock();
		unlink(path.c_str());
		conn_ctx.ipc->send(PSYNCNOTIFY_FILERECV_ERROR, PSYNCNOTIFY_IPC_NAME, path);
		throw;
	}
	sync_time=GetMTime();
	dav->Unlock();
	return 0;
}
sync_object* sync_object::getUpDir()
{
	sync_object* ret;
	ret=db->find(path.substr(0,path.rfind("/")));
	if(!ret)
		throw "Sync object parent search fail";
	return ret;
}

void sync_object::checkState(WebDav* dav)
{
	if(flags&FL_DELETE){
		state=SO_DELETE;
		return;
	}
	if(state!=SO_UPTODATE && 
			state!=SO_CONFLICT)
		return;
	sync_object srv=dav->GetServerState(uname);

	struct stat st;
	if(stat(path.c_str(),&st) && errno == ENOENT){//Нет у нас
		if(!level()){	
			state=SO_PULL;
			server_size=srv.server_size;
			mode=srv.mode;
			sTag=srv.sTag;
			if(!srv.valid())
				state=SO_CONFLICT; //нет ни у нас ни на сервере
			return;
		}else{ 
			if(getUpDir()->state == SO_PULL){
				state=SO_PULL;
				return;
			}
			flags|=FL_DELETE;
			if(!srv.valid())
				flags|=FL_DELETED; //Нет на сервере - ништяк)
			else
				state=SO_DELETE;	   //Удаляем и на сервере
			return;
		}
	}

	if(!srv.valid()){//Нет на сервере	//TODO: Вопрос - удален ли на сервере или создан у нас?
		if(!level())
			state=SO_CREATE;
		else{
			state=SO_CREATE;
			if(getUpDir()->state==SO_PULL ){
				state=SO_DELETE;
				flags|=FL_DELETE;
			}
		}
		return;
	}
	if(srv.sTag!=sTag){
		state=SO_PULL;
		server_size=srv.server_size;
		mode=srv.mode;
		sTag=srv.sTag;
		return;
	}
	if(sync_time==0){
		state=SO_PULL;
		return;
	}
	if(sync_time < GetMTime()){
		if(state!=SO_UPTODATE)
		{
			//TODO: Конфликт
			//state=SO_CONFLICT;
			//conn_ctx.ipc->send(ipc_proxy::NOTE_MSG, conn_ctx.pathNotify.c_str(),
			//	(string("Конфликт при обработке файла ")+path).c_str());
		}
		state=SO_PUSH;
	}
	return;
}

int sync_object::update(WebDav* dav)
{
	checkState(dav);
	cerr<<"Обработка "<< uname<<" ";
	//Удаление
	switch(state){
		case SO_DELETE:
			dav->Delete(uname);
			flags|=FL_DELETED;
			cerr<< "удален"<<endl;
			break;
		case SO_PUSH:
			cerr<<"> на сервер"<<endl;
			put(dav);
			break;
		case SO_PULL:
			cerr<<"< на локальную машину"<<endl;
			get(dav);
			break;
		case SO_CREATE:
			cerr<<"создаем на сервере"<<endl;
			put(dav);
			break;
		case SO_CONFLICT:
			cerr<<" conflict"<<endl;
			break;
		default:
			cerr<<" uptodate"<<endl;
			break;
	}
	if(S_ISDIR(mode) && !(flags&FL_DELETE))
		checkSubdir(dav);
	if(state!=SO_CONFLICT)
		state=SO_UPTODATE;
	return 1;
}

void sync_object::getData(string& _uname, string& _path,time_t& _sync_time,
		string& _stag, mode_t& _mode, char& _sync_point, char& _flags) const
{
	_uname=uname;
	_path=path;
	_sync_time=sync_time;			//Local
	_stag=sTag;
	_mode=mode;
	_sync_point=sync_point;
	_flags=flags;
}

static void write_string(stringstream& _s,const string s)
{
	unsigned sz=s.size();
	_s.write((char*)&sz,sizeof(sz));
	_s.write(s.c_str(),sz);
}

string sync_object::getStringData() const
{
	stringstream s;
	write_string(s, path);
	write_string(s, uname);
	write_string(s, sTag);
	s.write((char*)&sync_time,sizeof(sync_time));	
	s.write((char*)&mode,sizeof(mode));	
	s.write((char*)&flags,sizeof(flags));	
	s.write((char*)&sync_point,sizeof(sync_point)); 		
	return s.str();
}

map<string,string> sync_object::getMetaMap() const
{
	map<string,string> ret;
	ret["clientpath"]=path;
	ret["getstag"]=sTag;
	ostringstream m;
	m<<mode;
	ret["filemode"]=m.str();
	ostringstream _m;
	_m<<level();
	ret["level"]=_m.str();
	return ret;
}

int sync_object::valid()
{
	if(uname.empty())
		return 0;
	if(path.empty())
		return 0;
	if(mode==0)
		return 0;
	return 1;
}
